      subroutine add2cluster(good)
C
C This subroutine decides whether to add and adds, if appropriate, the newly 
C generated conformation whose energy is stored in ETOT and angles in ANGULO
C to the clusters. 
C The value of the logical variable good passed to the 
C subroutine indicates, what is the status of the new conformation: if .true.,
C it has been accepted and must be added to the set unconditionally, if .false.,
C it has been rejected by Metropolis procedure and the conformation is added
C only, if its energy is lower than the lowest energy of the conformation of
C the cluster it is similar to; it is rejected, if it doesn't belong to any
C of the clusters. On output, good=.true. indicates that the conformation 
C has been added.
C
C Used external functions and other objects:
C 
C   ICANT(I,J) - integer funtion - returns the "lower-diagonal" Cantor index
C        corresponding to indices I and J.
C
C   DIST_CONF(I,J) - real function - should return the distance between 
C        conformations I and J (optional, see below).
C
C   EXCH(I,J) - subroutine - exchanges two integers I and J.
C
C   UPDATE - subroutine - put the new conformation into the storage.
C
C   The ECEPPAK defines contained in the indicated INCLUDE files.
C
C   COMMON.CLUSTER - contains the defines needed for the storage of the
C        results of cluster analysis.
C
C Here is the explanation of the variables passed through the common blocks
C contained in COMMON.CLUSTER:
C
C NCON (I/O) - on entry the previous number of conformations, on output the 
C       number of conformations after possible addition of the new one.
C
C NGR (I/O) - on entry the number of clusters before adding the conformations
C       on output - the resulting number of clusters (1 < NGR < NCON) 
C       can actually become smaller, because some of the clusters can
C       merge after the new conformation is added.
C
C ICC(NCON) (I/O) - an array containing the numbers of conformations in such
C     an order that conformations of the same cluster occupy a contiguous 
C     region.
C
C ISTART(NGR) (I/O) - an array containing the numbers of the elements of ICC
C     that start a new cluster. 
C
C NMERGED - the number of clusters that have merged; includes the new 
C     conformation considered as a previously separated cluster.
C
C IMERGED(NMERGED) - the OLD numbers of clusters that have merged upon adding
C     the new conformation.
C
C ITRACK(NGR) - shows how the new numbers of clusters relate to the old ones. 
C
C ICURRENT (O) - the number of the cluster which was created or updated by the 
C     routine.
C
C ATTALUMS(NCON*(NCON-1)/2) - contains the lower-diagonal of the matrix of
C     the distances between conformations; the position of the distance i,j
C     (j<i) in the array is given by the Cantor index: I = (i-1)(i-2)/2+j.
C     This array can be pre-computed or updated dynamically within the 
C     subroutine. In the latter case uncomment the line with the call to the
C     function dist_conf(i,j) that should return the distance between 
C     conformations i and j and supply this function.
C
C A. Liwo,  5/7/97
C D. Ripoll, 5/14/97
C
C Bugs to: adam@sun1.chem.univ.gda.pl, adam@scheraga1.chem.cornell.edu
C
#ifdef PHCALC
      INCLUDE 'PARAMET.ECEPPH'
#else
      INCLUDE 'PARAMET.ECEPP'
#endif
      include 'COMMON.IOUNIT'
      include 'COMMON.CNTRL'
      include 'COMMON.MYCONF'
      include 'COMMON.CALC'
      include 'COMMON.TRACK'
      include 'COMMON.CLUSTER'
      INCLUDE 'COMMON.EDMC'
      INCLUDE 'COMMON.NNDRUK'
      logical good
      integer ibuffer(maxnum_track)
      parameter (num_in_line=5)

c     print *,'add2cluster: good=',good,' ncon=',ncon
c if the energy is higher than that of the lowest-energy conformation of cluster 11: forget it!
      if (.not.good .and. etot.gt.(enemin(icc(1))+5.00) ) return  
      do i=1,ngr
        itrack(i)=i
      enddo
C Calculate the distance of the current from the preceding confs.
C Uncomment the following lines and supply dist_conf, if you want to update
C the distance array dynamically.
      call dist_new_conf(rms_min,icnfn)
      if (rms_min.lt.0.1) then
c        write(jout,'(a,i5)')'  rms: conf. found equal to ',icnfn 
c        write(*,'(a,i5)')' rms: conf. found equal to',icnfn 
         ediff= enemin(icnfn)- etot 
c        write(jout,'(a,2e12.5)') 'ENERGIES', enemin(icnfn), etot 
c        write(*,'(a,2e12.5)') 'ENERGIES', enemin(icnfn), etot 
!        if (ediff.lt.ecrit  .and. ediff.gt.-ecrit1) then
c           write(*,'(a,3e12.5)') 
c    &        ' SIMILAR: -ecrit1 ediff   ecrit  ', -ecrit1,ediff,ecrit 
C Conformations have same rms and same energy             
            if (good) then 
c             write (*,'(a,3(a,i5))') 
c    &        'Conformation accepted by Metropolis,',
c    &        ' but found similar to conformation',
c    &        icnfn,' from iteration ',niterat(icnfn),
c    &        ' cluster ',icluster(icnfn)

c             write (jout,'(a,3(a,i5))') 
c    &        'Conformation accepted by Metropolis,',
c    &        ' but found similar to conformation',
c    &        icnfn,' from iteration ',niterat(icnfn),
c    &        ' cluster ',icluster(icnfn)
              if (.not.laccept(icnfn)) then
                 niterat(icnfn)=ndone
c                write (*,'(a,i5)') 'niterat set to=', niterat(icnfn)
              endif 
              laccept(icnfn)=.true.
!             write (jout,*) '1 laccept(', icnfn,')=',laccept(icnfn)
              ncount(icnfn)=ncount(icnfn)+1
              ici = icc(icnfn)
              mst = icnfn
              icurrent = icluster(icnfn)
            else
c             write (jout,'(a,3(a,i5))') 
c    &        'Conformation rejected by Metropolis',
c    &        ' also was found similar to conformation',
c    &        icnfn,' from iteration ',niterat(icnfn),
c    &        ' cluster ',icluster(icnfn)
c             write (*,'(a,3(a,i5))') 
c    &        'Conformation rejected by Metropolis',
c    &        ' also was found similar to conformation',
c    &        icnfn,' from iteration ',niterat(icnfn),
c    &        ' cluster ',icluster(icnfn)
            endif
            return
!        else
!           write(*,'(a,i5)')' rms: conf. found similar to',icnfn 
!           write(*,'(2a/,3e12.5)') 'But DIFFERS in Energy',
!    &      ' -ecrit1 - ediff - ecrit',-ecrit1, ediff,ecrit 
!           write(*,'(a)')   'WILL BE ADDED TO A CLUSTER'
!        endif
      endif

C Find the place of the new conf in the set, acc. to distance.
      nmerged=0
      do igr=1,ngr
C Get the pointers to the beginning and the end of igr-th cluster.
        is = istart(igr)
        ie = istart(igr+1)-1
C Loop through the cluster.
        do j=is,ie
          icj = icc(j)
          if (attalums(icant(icj,ncon+1)).lt.rcut) then
            nmerged=nmerged+1
            imerged(nmerged)=igr
            goto 10
          endif
        enddo !j
   10   continue 
      enddo !i  
      if (nmerged.gt.0) goto 20
C nmerged=0 means that the new conformation is isolated from the other ones.
C Just add a new family, if the conformation is "good".
      if (good) then
        write (jout,'(a)') 'New cluster created.'
        ngr=ngr+1
        ncon=ncon+1
        icc(ncon)=ncon
        istart(ngr)=ncon
        istart(ngr+1)=ncon+1
C Store the added conformation
        call update(good)
        if (.not. not_done) return
        ncount(ncon)=1
        laccept(ncon)=.true.
!       write (jout,*) '2 laccept(', ncon,')=',laccept(ncon)
        niterat(ncon)=ndone
        itrack(ngr)=ngr
        nmerged=1
        imerged(1)=ngr
        goto 50
      endif
      return
C The conformation is to be incorporated in one of the so-far existing clusters.
C Some of the clusters might also merge.
   20 if (good) goto 30
      laccept(ncon+1)=.false.
!     write (jout,*) '3 laccept(', ncon+1,')=',laccept(ncon+1)
      do i=1,nmerged
        if (enemin(icc(istart(i))).gt.etot) goto 30
      enddo
C If this point is reached, the conformation is no better than the lowest energy
C conformation of any cluster it belongs to.
      return
   30 ngr=ngr+1
      ncon=ncon+1
      if (good) then
         laccept(ncon)=.true.
!        write (jout,*) '4 laccept(', ncon,')=',laccept(ncon)
         niterat(ncon)=ndone
      endif
      icc(ncon)=ncon
      istart(ngr)=ncon
      istart(ngr+1)=ncon+1 
      nmerged=nmerged+1
      imerged(nmerged)=ngr
C Store the added conformation
      call update(good)
      if (good) ncount(ncon)=1
C     print *,'mst=',mst
      if (.not. not_done) return
      good=.true.
      itrack(ngr)=ngr
      ipos=0
      do i=1,nmerged
C Copy the elements of the merged clusters 1...nmerge to the temporary array 
C ibuffer.
        do j=istart(imerged(i)),istart(imerged(i)+1)-1
          ipos=ipos+1
          ibuffer(ipos)=icc(j)
        enddo ! j
      enddo !i
C Up-shift the remaining clusters, in order to make room for the merged cluster.
      ish=istart(imerged(nmerged)+1)-istart(imerged(nmerged))
      jup=imerged(nmerged)
      do i=nmerged-1,1,-1
        jlow=imerged(i)+1
        iish=nmerged-i
        do j=jup-1,jlow,-1
          do k=istart(j+1)-1,istart(j),-1
            icc(k+ish)=icc(k)
          enddo !k
          istart(j+iish)=istart(j)+ish
          itrack(j+iish)=itrack(j)
        enddo !j
        jup=jlow-1
        ish=ish+istart(imerged(i)+1)-istart(imerged(i))
      enddo !i
C Copy the merged cluster from ibuffer back to icc.
      ii1=imerged(1)
      istart(ii1+1)=istart(ii1)+ipos
      is1=istart(ii1)
      do i=0,ipos-1
        icc(i+is1)=ibuffer(i+1)
      enddo !i
      ngr=ngr-nmerged+1
      do i=ii1+2,ngr+1
        istart(i)=istart(i+nmerged-1)
        itrack(i)=itrack(i+nmerged-1)
      enddo !i
C Energy-sort the confs in the updated cluster.
      is2=istart(ii1+1)-1
      do i=is1,is2
        ici=icc(i)
        emin=enemin(ici)
        imin=i
        do j=i+1,is2
          icj=icc(j)
          enej=enemin(icj)
          if (emin.gt.enej) then
            emin=enej
            imin=j
          endif
        enddo !j
        if (imin.ne.i) call exch(icc(i),icc(imin))
      enddo !i
   50 continue
C Energy-sort the clusters.
      is=imerged(1)
      icurrent=is
      emin=enemin(icc(istart(is)))
      do il=is-1,1,-1
        if (enemin(icc(istart(il))).le.emin) goto 40
      enddo !il
C The updated cluster's energy is not lower than the energy of the preceding
C cluster... Nothing to do more.
C  40 if (il.eq.is-1) return
   40 if (il.ne.is-1) then
C The energy of the leading member of the updated cluster found lower than 
C the energy of il+1-th preceding cluster, but higher than that of cluster il. 
C So, rearrange the clusters. First, copy the updated cluster to the IBUFFER
C array.
         ish=istart(is+1)-istart(is)
         do i=istart(is),istart(is+1)-1
        ibuffer(i-istart(is)+1)=icc(i)
         enddo !i
         itrackis=itrack(is)
C Up-shift the clusters il,..,is-1 to make room for the is-th cluster at 
C position il.
         do i=is-1,il+1,-1
           do j=istart(i+1)-1,istart(i),-1
             icc(j+ish)=icc(j)
           enddo !j
           istart(i+1)=istart(i)+ish
           itrack(i+1)=itrack(i)
         enddo !i
C Copy the updated cluster to position il.
         itrack(il+1)=itrackis
         isil=istart(il+1)
         do i=isil,istart(il+2)-1
           icc(i)=ibuffer(i-isil+1)
         enddo !i
         icurrent=il+1
      endif
C Assign clusters to conformations.
      do i=1,ngr
        do j=istart(i),istart(i+1)-1
          icluster(icc(j))=i
        enddo
      enddo
      if (nprint_elec.gt.0) then
        nprm=nprint_elec*int(ndone/nprint_elec)
        if (nprm.eq.ndone) nuk=1
      endif
      if(nuk.eq.1) then
         write (jout,100) ngr
         do igr=1,ngr
           ilicz=istart(igr+1)-istart(igr)
           write (jout,200) igr,ilicz
           nrecord=ilicz/num_in_line
           ind1=istart(igr)
           do irecord=1,nrecord
             ind2=ind1+num_in_line-1
             write (jout,300) (icc(ico),enemin(icc(ico)),
     &              ico=ind1,ind2)
             ind1=ind2+1
           enddo
           write (jout,300) (icc(ico),enemin(icc(ico)),
     &              ico=ind1,istart(igr+1)-1)
         enddo
         write (jout,400)
         write (jout,500) (i,icluster(i),i=1,ncon)
      endif
  100 FORMAT (//'THERE ARE ',I4,' FAMILIES OF CONFORMATIONS')
  200 FORMAT (/'FAMILY ',I4,' CONTAINS ',I4,' CONFORMATION(S): ')
  300 FORMAT (5(I4,1pe12.4))
  400 FORMAT (//'ASSIGNMENT OF CONSECUTIVE CONFORMATIONS TO FAMILIES:')
  500 FORMAT (7(2I5,1X))
      return
      end
c------------------------------------------------------------------------------------------------
      subroutine select_from_cluster
#ifdef PHCALC
      INCLUDE 'PARAMET.ECEPPH'
#else
      INCLUDE 'PARAMET.ECEPP'
#endif
      include 'COMMON.IOUNIT'
      include 'COMMON.CNTRL'
      include 'COMMON.MYCONF'
      include 'COMMON.CALC'
      include 'COMMON.TRACK'
      include 'COMMON.CLUSTER'
      INCLUDE 'COMMON.EDMC'
      INCLUDE 'COMMON.GEN100'
      INCLUDE 'COMMON.BONDSX'
      logical not_found, fail 
      data deltemx /2.0/

      fail = .false. 
      if (icurrent.gt.1) then
        emin = enemin(icc(1))
      else 
        emin = enemin(icc(istart(2)))
      endif

C see if including the volume of the cluster (as in entropy sampling) helps
      sumprobcl=0.0D0
      do i=1,ngr
         ivolcls= istart(i+1)-istart(i)
! the relative volume of the cluster is kvolclls/ncon
         relvolp_cl(i)= 1.0-float(ivolcls)/float(ncon)
         sumprobcl=sumprobcl+relvolp_cl(i)
      enddo
      sumprobcl_inv=1.0E0/sumprobcl
! renormalize probabilities
      do i=1,ngr
         tmpn=relvolp_cl(i)
         relvolp_cl(i)=tmpn*sumprobcl_inv 
      enddo

C chose cluster
      ncont=0
      not_found = .true.
      do while (not_found)
        iclust = iran_num(1,ngr)
        if (iclust.ne.icurrent) then
          delte = (enemin(icc(istart(iclust))) - emin)/(1.2*inumrs)
          if (delte.gt.deltemx) goto 10
C
C         expe = exp (-delte)
C    include the entropy sampling probability  
          expe = exp (-delte) *relvolp_cl(iclust)
C
          rannu = ran_number(0.0,1.0)
          if (expe.ge.rannu) not_found = .false. 
        endif
   10   continue
        ncont=ncont+1
        if (ncont.ge.10000) then 
          not_found = .false. 
          fail = .true. 
        endif
      enddo
      if (fail) Then
         write(*,*) ' select_from cluster: Failed at 44675'
         call sleep(10)
         stop 44675
      endif


C  chose conformation from selected cluster
      ncont=0
      not_found = .true.
      emin = enemin(icc(istart(iclust))) 
      do while (not_found)
        icon = iran_num(istart(iclust),istart(iclust+1)-1)
        delte = (enemin(icc(icon)) - emin)/(1.2*inumrs)
        if (delte.gt.deltemx) goto 20
        expe = exp (-delte)
        rannu = ran_number(0.0,1.0)
        if (expe.ge.rannu) not_found = .false.
   20   continue
        ncont=ncont+1
        if (ncont.ge.10000) then 
          not_found = .false. 
          fail = .true. 
        endif
      enddo
      if (fail) Then
         write(*,*) ' select_from cluster: Failed at 44676'
         call sleep(10)
         stop 44676
      endif

      ii = icc(icon)

      write (jout,'(a,i4,a,i4)') 'Selected cluster',iclust,
     &   ' conformation',ii
      if (onscreen) write (*,'(a,i4,a,i4)') 'Selected cluster',iclust,
     &   ' conformation',ii
C  SET UP ANGL ARRAY AND CODE, USING MOST RECENT MINIMUM
       DO 25 I=1,INUMRS
         DO 26 j=1,maxnum_dihe
            ANGULO(J,I)=ANGMIN(J,I,II)
   26    CONTINUE
   25  CONTINUE
C changes in next line are needed
       LPRHI=LPROHIS(II)

      return
      end
