C $Date: 1997/03/11 17:33:29 $
C $Revision: 3.2 $
C
C
C
      SUBROUTINE ORIENT1(X1,X2,IATMCD,UX1)
* This subroutine orients the new protein fragment X1 so as to
* minimize the rms with X2.  The subroutine first tries the lagrange
* multiplier method of Kabsch (acta cryst A32 922) and if this fails,
* direct minimization is used.  In Kabsch's method, a matrix U is
* applied to X1 to give new coords UX1.  The nine elements of U are
* viewed as parameters.  The rms between UX1 and X2 is minimized subject
* to the constraint that U be orthogonal, and this leaves only three
* of the parameters independent, as we expect.  The constraint is
* applied by introducing nine lagrange multipliers, one for each element
* of the orthogonality condition UtU=1.  Then we minimize by setting the
* first deriv of the object function to zero.  The result of this is that
*              U = R * (RTR)**(-1/2)
* where R(i,j) is gotten by summing X2(i)*X1(j) over all atoms, and RTR
* R premultiplied by R transposed.  RTR is raised to the minus one half
* power by the spectral theorem.  Note that this doesn't work if any
* eigvals are negative so I test for this in the program.  Theoretically
* RTR is positive definite, but in practice errors occur.  Note also
* that U is not constrained to have det=+1, and in practice det=-1
* occurs frequently.  For neg eigvals and mirror inversion I use a
* simple minimization which, occasionally, fails to converge.

      IMPLICIT REAL*8 ( A-H,O-Z)
#ifdef PHCALC
      INCLUDE 'PARAMET.ECEPPH'
#else
      INCLUDE 'PARAMET.ECEPP'
#endif
      INTEGER I,J,K,L,IATOM,IATMCD,IER
      DIMENSION U(1:3,1:3),R(1:3,1:3),RTR(1:3,1:3),
     &       EIGVAL(1:3),T(1:3,1:3),UX1(1:3,1:IATMCD),
     &       X1(1:3,1:IATMCD),X2(1:3,1:IATMCD),WK(1:9)
      REAL*8 X1_AL(3,maxnum_atoms),X2_AL(3,maxnum_atoms),
     &      UX1_AL(3,maxnum_atoms)
      COMMON /X1ALL/ X1_AL,UX1_AL, X2_AL, NATMOM
c*ksr* PSC /X1ALL/
      INCLUDE 'COMMON.IOUNIT'
 
      IF(IATMCD.EQ.0) THEN
         WRITE(JOUT,5550) IATMCD
5550     FORMAT('ERROR  IN  SUBROUTINE orient- IATMCD = ',I7)
         STOP 5550
      ENDIF
      IF(NATMOM.EQ.0) THEN
         WRITE(JOUT,5551) NATMOM
5551     FORMAT('ERROR  IN  SUBROUTINE orient- NATMOM = ',I7)
         STOP  5551
      ENDIF
      DO 109 I=1,3
C  FOR PEPLEN LARGE, SHOULD CHANGE LOOP ORDER
        DO 110 J=1,3
C  NOTE THAT R IS NOT SYMMETRIC
          R(I,J)=0.0
          DO 111 IATOM=1,IATMCD
            R(I,J)=R(I,J)+X2(I,IATOM)*X1(J,IATOM)
111       CONTINUE
110     CONTINUE
109   CONTINUE
      DO 139 I=1,3
        DO 140 J=1,3
          RTR(I,J)=0.0
          DO 141 K=1,3
            RTR(I,J)=RTR(I,J)+R(K,I)*R(K,J)
141       CONTINUE
140     CONTINUE
139   CONTINUE
C
C     CALL EIGRS (RTR,3,12,EIGVAL,T,3,WK,IER)
C
      IJUMP=0
#ifdef AIX
      CALL DEVCSF (3,RTR,3,EIGVAL,T,3)
      IJUMP=1
#endif
#ifdef ES9000
      CALL DEVCSF (3,RTR,3,EIGVAL,T,3)
      IJUMP=1
#endif
      IF (IJUMP.EQ.0) THEN
           WRITE ( JOUT, '(A)')
     & 'CHOOSE GOLUB METHOD SINCE IMSL ROUTINES ARE NOT AVAILABLE'
           STOP
      ENDIF
 
C     WRITE ( JOUT, '(3E15.6)') EIGVAL(1), EIGVAL(2), EIGVAL(3)
      DO 150 I=1,3
        IF (EIGVAL(I).LT.0.0) CALL ERR(2)
C***************************************************************************
C  TEST FOR NEG EIGVALS
C***************************************************************************
        EIGVAL(I)=1.0/DSQRT(EIGVAL(I))
150   CONTINUE
      DO 158 I=1,3
C       NOW CALCULATE ROT MATRIX
        DO 159 J=1,3
          U(I,J)=0.0
          DO 160 K=1,3
            DO 161 L=1,3
              U(I,J)=U(I,J)+R(I,K)*T(K,L)*EIGVAL(L)*T(J,L)
161         CONTINUE
160       CONTINUE
159     CONTINUE
158   CONTINUE
      DET=U(1,1)*(U(2,2)*U(3,3)-U(3,2)*U(2,3))
     &   -U(1,2)*(U(2,1)*U(3,3)-U(3,1)*U(2,3))
     &   +U(1,3)*(U(2,1)*U(3,2)-U(3,1)*U(2,2))
      IF (DET.LT.0.0) CALL ERR(3)
C***************************************************************************
C       TEST FOR INVERSION
C***************************************************************************
      DO 169 IATOM=1,IATMCD
C      NOW TRANSFORM THE COORDS
        DO 170 I=1,3
        UX1(I,IATOM)=0.0
          DO 171 J=1,3
            UX1(I,IATOM)=UX1(I,IATOM)+U(I,J)*X1(J,IATOM)
171       CONTINUE
170     CONTINUE
169   CONTINUE
C**********************DR 10/90******************************
c     WRITE ( JOUT, 1110)
c1110  FORMAT('Everything is OK till now')
      DO 200 IA=1,NATMOM
C   Now transform the coordinates of all the atoms, including those
C   that were not used in the fitting.
         DO 210 I=1,3
              UX1_AL(I,IA)=0.0
              DO 220 J=1,3
                   UX1_AL(I,IA)=UX1_AL(I,IA)+U(I,J)*X1_AL(J,IA)
220           CONTINUE
210      CONTINUE
200   CONTINUE
C**********************DR 10/90******************************
      RETURN
      END
