      real function prng_next(me)
      logical prng_restart, prng_chkpnt
c
c Calling sequence: 
c      <new random number> = prng_next ( <ordinal of generator desired> )
c
c This code is based on a sequential algorithm provided by Mal Kalos.
c This version uses 4 16-bit packets, and uses a block data common
c area for the initial seeds and constants.  A 64-bit floating point
c number is returned.
c
c The arrays "l" and "n" are full-word aligned, being padded by zeros
c That is, rows 1-4 in a given column are for real, rows 5-16 are bogus
c
c July 12, 1993: double the number of sequences.  We should have been
c                using two packets per seed, rather than four
c
      real tpm12
      integer iseed(4)
      parameter(tpm12 = 1.d0/65536.d0)
      parameter(nmax=1021)
c     external prngblk
      common/ksrprng/l(16,0:nmax),n(16,0:nmax)
c*ksr*subpage /ksrprng/
      data m1,m2,m3,m4 / 0, 8037, 61950, 30779/
      if (me .lt. 0 .or. me .gt. nmax) then
         prng_next=-1.0
	 return
      endif
      l1=l(1,me)
      l2=l(2,me)
      l3=l(3,me)
      l4=l(4,me)
      i1=l1*m4+l2*m3+l3*m2+l4*m1 + n(1,me)
      i2=l2*m4+l3*m3+l4*m2 + n(2,me)
      i3=l3*m4+l4*m3 + n(3,me)
      i4=l4*m4 + n(4,me)
      l4=and(i4,65535)
      i3=i3+ishft(i4,-16)
      l3=and(i3,65535)
      i2=i2+ishft(i3,-16)
      l2=and(i2,65535)
      l1=and(i1+ishft(i2,-16),65535)
      prng_next=tpm12*(l1+tpm12*(l2+tpm12*(l3+tpm12*l4)))
      l(1,me)=l1
      l(2,me)=l2
      l(3,me)=l3
      l(4,me)=l4
      return
c
c   prng_chkpnt          Get the current state of a generator
c
c Calling sequence:
c   logical prng_chkpnt, status
c   status = prng_chkpnt (me, iseed)    where
c
c     me is the particular generator whose state is being gotten
c     seed is an 4-element integer array where the "l"-values will be saved
c
      entry prng_chkpnt (me, iseed)
      if (me .lt. 0 .or. me .gt. nmax) then
         prng_chkpnt=.false.
      else
	 prng_chkpnt=.true.
	 iseed(1)=l(1,me)
	 iseed(2)=l(2,me)
	 iseed(3)=l(3,me)
	 iseed(4)=l(4,me)
      endif
      return
c
c   prng_restart          Restart generator from a saved state
c
c Calling sequence:
c   logical prng_restart, status
c   status = prng_restart (me, iseed)    where
c
c     me is the particular generator being restarted
c     seed is an 4-element integer array containing the "l"-values
c
      entry prng_restart (me, iseed)
      if (me .lt. 0 .or. me .gt. nmax) then
         prng_restart=.false.
	 return
      else
	 prng_restart=.true.
	 l(1,me)=iseed(1)
	 l(2,me)=iseed(2)
	 l(3,me)=iseed(3)
	 l(4,me)=iseed(4)
      endif
      return
      end
