      SUBROUTINE GOLUB (  RMS, FAIL )
C This routine is a driver to calculate rms deviations using the
C procedure of Singular Value Decomposotion described in "MATRIX COMPUTATIONS" 
C by Gene H. Golub and Charles F. Van Loan. The Johns Hopkins University Press.
C 19??)  pp.425-431.
C  rms routines (FITSQ, MVVAD and calling routines) were written by Ken Gibson.
C

      IMPLICIT REAL*8 ( A-H,O-Z)
#ifdef PHCALC
      INCLUDE 'PARAMET.ECEPPH'
#else
      INCLUDE 'PARAMET.ECEPP'
#endif
      DIMENSION  T(3),B(3,3)
      COMMON /REA/ REAN(3)
      INCLUDE 'COMMON.RMSFIT'
      INCLUDE 'COMMON.IOUNIT'
      LOGICAL FAIL
      
      FAIL=.FALSE.

C**********************DR 3/94******************************
      IF(IATMCD.EQ.0) THEN
         WRITE(JOUT,1500) IATMCD
1500     FORMAT('ERROR  IN  SUBROUTINE GOLUB  - IATMCD = ',I7)
         FAIL=.TRUE.
         RETURN
      ENDIF
C  
C ***********RESET ORIGIN OF VECTOR COORDINATES TO CENTER OF MASS.******
      CALL COM(X1,IATMCD)
      IF(NATMOM.EQ.0) THEN
         WRITE(JOUT,5551) NATMOM
5551     FORMAT('ERROR IN GOLUB  - ZERO ATOMS; NATMOM = ',I7)
         FAIL=.TRUE.
c        STOP  5551
      ENDIF
      DO 240 IJ=1,NATMOM
         DO 241 IK=1,3
            X1_AL(IK,IJ)=X1_AL(IK,IJ)-REAN(IK)
241      CONTINUE
240   CONTINUE

      CALL COM(X2,IATMCD)
      DO 540 IJ=1,NATMOM
         DO 541 IK=1,3
            X2_AL(IK,IJ)=X2_AL(IK,IJ)-REAN(IK)
541      CONTINUE
540   CONTINUE

C ************************************************************

C     CALL FITSQ(X2,X1,IATMCD,T,B,RMS)
      CALL FITSQ(X1,X2,IATMCD,T,B,RMS,FAIL)
      IF (FAIL) THEN
         RMS=0.0E0
         FAIL=.FALSE.
      ENDIF
        

C PRINT TRANSLATION VECTOR
C     WRITE (JOUT,1600) (T(II),II=1,3)
1600  FORMAT (/,' T = ',3F15.7)
C PRINT ROTATION MATRIX
C     WRITE (JOUT,*) ' B = '
C     DO 10 I=1,3
C10   WRITE (JOUT,1700) (B(I,J),J=1,3)
1700  FORMAT (5X,3F15.7)

C ROTATE AND TRANSLATE SECOND SETS OF COORDINATES
C      DO 20 I=1,IATMCD
C         CALL MVVAD(B,X1(1,I),T,UX1(1,I))
C         DO 741 IK=1,3
C           UX1(IK,I)=UX1(IK,I)+REAN(IK)
C741      CONTINUE
C
C 20   CONTINUE

C ROTATE SECOND SETS OF COORDINATES
      DO 20 I=1,IATMCD
         CALL MVVAD(B,X1(1,I),T,UX1(1,I))
 20   CONTINUE


C      DO 30 I=1,NATMOM
C         CALL MVVAD(B,X1_AL(1,I),T,UX1_AL(1,I))
C         DO 751 IK=1,3
C            UX1_AL(IK,I)=UX1_AL(IK,I)+REAN(IK)
C751      CONTINUE
C 30   CONTINUE

      DO 30 I=1,NATMOM
         CALL MVVAD(B,X1_AL(1,I),T,UX1_AL(1,I))
 30   CONTINUE


C     WRITE (JOUT,17) RMS
C17   FORMAT ('THE RMS FROM FITSQ METHOD IS ',F12.8)

      RETURN
      END
      SUBROUTINE FITSQ(X,Y,NN,T,B,RMS, FAIL)
C  X AND Y ARE THE VECTORS OF COORDINATES (DIMENSIONED 1 TO 3*NN) OF THE
C  STRUCTURES TO BE SUPERIMPOSED (WITH NN EQUAL TO THE NUMBER OF POINTS
C  IN EACH).   T AND B ARE RESPECTIVELY THE TRANSLATION VECTOR AND THE
C  ROTATION MATRIX THAT TRANSFORMS THE FIRST SET OF COORDINATES TO THE
C  FRAME OF THE SECOND SET.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'COMMON.IOUNIT'

C     DIMENSION X(1),Y(1),T(3)
      DIMENSION X(3*NN),Y(3*NN),T(3)
      DIMENSION B(3,3),Q(3,3),R(3,3),V(3),XAV(3),YAV(3),E(3),C(3,3)
      DATA SMALL /1.0D-10/
      LOGICAL FAIL
      
C     FAIL=.FALSE.
      FN=1.0/NN
      DO 10 I=1,3
      XAV(I)=0.0
      YAV(I)=0.0
      DO 10 J=1,3
   10 B(J,I)=0.0
      NC=0
      DO 30 N=1,NN
      DO 20 I=1,3
      XAV(I)=XAV(I)+X(NC+I)*FN
   20 YAV(I)=YAV(I)+Y(NC+I)*FN
   30 NC=NC+3
      NC=0
      RMS=0.0
      DO 50 N=1,NN
      DO 40 I=1,3
      RMS=RMS+(X(NC+I)-XAV(I))**2+(Y(NC+I)-YAV(I))**2
      DO 40 J=1,3
      B(J,I)=B(J,I)+(X(NC+I)-XAV(I))*(Y(NC+J)-YAV(J))
   40 C(J,I)=B(J,I)
   50 NC=NC+3
      CALL SIVADE(B,Q,R,D)
      SN3=SIGN(1.0D0,D)
      DO 120 I=1,3
      DO 120 J=1,3
  120 B(J,I)=-Q(J,1)*R(I,1)-Q(J,2)*R(I,2)-SN3*Q(J,3)*R(I,3)
      CALL MVVAD(B,XAV,YAV,T)
      DO 130 I=1,3
      DO 130 J=1,3
      RMS=RMS+2.0*C(J,I)*B(J,I)
  130 B(J,I)=-B(J,I)
      RMS=FN*RMS
      IF (ABS(RMS).GT.SMALL) GO TO 140
      RMS=0.0
      WRITE (JOUT,301)
      RETURN
  140 IF (RMS.GT.0.0 .or. rms.gt.-0.001) GO TO 150
      WRITE (JOUT,303) RMS
C      STOP 'negative RMS deviation'
      WRITE(*,*) 'negative RMS deviation'
      FAIL=.TRUE.
  150 RMS=SQRT(abs(RMS))
C     WRITE (JOUT,302) RMS
      RETURN
  301 FORMAT (5X,'RMS DEVIATION NEGLIGIBLE')
  302 FORMAT (5X,'RMS DEVIATION ',F15.5)
  303 FORMAT (//,5X,'NEGATIVE MS DEVIATION - ',F14.6)
      END
      SUBROUTINE SIVADE(X,Q,R,DT)
C  COMPUTES Q,E AND R SUCH THAT Q(T)XR = DIAG(E)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'COMMON.IOUNIT'
      DIMENSION X(3,3),Q(3,3),R(3,3),E(3)
      DIMENSION H(3,3),P(3,3),U(3,3),D(3)
      DATA SMALL /1.0D-16/
      XNRM=0.0
      DO 20 I=1,3
      DO 10 J=1,3
      XNRM=XNRM+X(J,I)*X(J,I)
      U(J,I)=0.0
      R(J,I)=0.0
   10 H(J,I)=0.0
      U(I,I)=1.0
   20 R(I,I)=1.0
      XNRM=SQRT(XNRM)
      DO 110 N=1,2
      XMAX=0.0
      DO 30 J=N,3
   30 IF (ABS(X(J,N)).GT.XMAX) XMAX=ABS(X(J,N))
      A=0.0
      DO 40 J=N,3
      H(J,N)=X(J,N)/XMAX
   40 A=A+H(J,N)*H(J,N)
      A=SQRT(A)
      DEN=A*(A+ABS(H(N,N)))
      D(N)=1.0/DEN
      H(N,N)=H(N,N)+SIGN(A,H(N,N))
      DO 70 I=N,3
      S=0.0
      DO 50 J=N,3
   50 S=S+H(J,N)*X(J,I)
      S=D(N)*S
      DO 60 J=N,3
   60 X(J,I)=X(J,I)-S*H(J,N)
   70 CONTINUE
      IF (N.GT.1) GO TO 110
      XMAX=MAX(ABS(X(1,2)),ABS(X(1,3)))
      H(2,3)=X(1,2)/XMAX
      H(3,3)=X(1,3)/XMAX
      A=SQRT(H(2,3)*H(2,3)+H(3,3)*H(3,3))
      DEN=A*(A+ABS(H(2,3)))
      D(3)=1.0/DEN
      H(2,3)=H(2,3)+SIGN(A,H(2,3))
      DO 100 I=1,3
      S=0.0
      DO 80 J=2,3
   80 S=S+H(J,3)*X(I,J)
      S=D(3)*S
      DO 90 J=2,3
   90 X(I,J)=X(I,J)-S*H(J,3)
  100 CONTINUE
  110 CONTINUE
      DO 130 I=1,3
      DO 120 J=1,3
  120 P(J,I)=-D(1)*H(J,1)*H(I,1)
  130 P(I,I)=1.0+P(I,I)
      DO 140 I=2,3
      DO 140 J=2,3
      U(J,I)=U(J,I)-D(2)*H(J,2)*H(I,2)
  140 R(J,I)=R(J,I)-D(3)*H(J,3)*H(I,3)
      CALL MMMUL(P,U,Q)
  150 NP=1
      NQ=1
      IF (ABS(X(2,3)).GT.SMALL*(ABS(X(2,2))+ABS(X(3,3)))) GO TO 160
      X(2,3)=0.0
      NQ=NQ+1
  160 IF (ABS(X(1,2)).GT.SMALL*(ABS(X(1,1))+ABS(X(2,2)))) GO TO 180
      X(1,2)=0.0
      IF (X(2,3).NE.0.0) GO TO 170
      NQ=NQ+1
      GO TO 180
  170 NP=NP+1
  180 IF (NQ.EQ.3) GO TO 310
      NPQ=4-NP-NQ
      IF (NP.GT.NPQ) GO TO 230
      N0=0
      DO 220 N=NP,NPQ
      NN=N+NP-1
      IF (ABS(X(NN,NN)).GT.SMALL*XNRM) GO TO 220
      X(NN,NN)=0.0
      IF (X(NN,NN+1).EQ.0.0) GO TO 220
      N0=N0+1
      GO TO (190,210,220),NN
  190 DO 200 J=2,3
  200 CALL GIVNS(X,Q,1,J)
      GO TO 220
  210 CALL GIVNS(X,Q,2,3)
  220 CONTINUE
      IF (N0.NE.0) GO TO 150
  230 NN=3-NQ
      A=X(NN,NN)*X(NN,NN)
      IF (NN.GT.1) A=A+X(NN-1,NN)*X(NN-1,NN)
      B=X(NN+1,NN+1)*X(NN+1,NN+1)+X(NN,NN+1)*X(NN,NN+1)
      C=X(NN,NN)*X(NN,NN+1)
      DD=0.5*(A-B)
      XN2=C*C
      RT=B-XN2/(DD+SIGN(SQRT(DD*DD+XN2),DD))
      Y=X(NP,NP)*X(NP,NP)-RT
      Z=X(NP,NP)*X(NP,NP+1)
      DO 300 N=NP,NN
      IF (ABS(Y).LT.ABS(Z)) GO TO 240
      T=Z/Y
      C=1.0/SQRT(1.0+T*T)
      S=C*T
      GO TO 250
  240 T=Y/Z
      S=1.0/SQRT(1.0+T*T)
      C=S*T
  250 DO 260 J=1,3
      V=X(J,N)
      W=X(J,N+1)
      X(J,N)=C*V+S*W
      X(J,N+1)=-S*V+C*W
      A=R(J,N)
      B=R(J,N+1)
      R(J,N)=C*A+S*B
  260 R(J,N+1)=-S*A+C*B
      Y=X(N,N)
      Z=X(N+1,N)
      IF (ABS(Y).LT.ABS(Z)) GO TO 270
      T=Z/Y
      C=1.0/SQRT(1.0+T*T)
      S=C*T
      GO TO 280
  270 T=Y/Z
      S=1.0/SQRT(1.0+T*T)
      C=S*T
  280 DO 290 J=1,3
      V=X(N,J)
      W=X(N+1,J)
      A=Q(J,N)
      B=Q(J,N+1)
      X(N,J)=C*V+S*W
      X(N+1,J)=-S*V+C*W
      Q(J,N)=C*A+S*B
  290 Q(J,N+1)=-S*A+C*B
      IF (N.GE.NN) GO TO 300
      Y=X(N,N+1)
      Z=X(N,N+2)
  300 CONTINUE
      GO TO 150
  310 DO 320 I=1,3
  320 E(I)=X(I,I)
  330 N0=0
      DO 360 I=1,3
      IF (E(I).GE.0.0) GO TO 350
      E(I)=-E(I)
      DO 340 J=1,3
  340 Q(J,I)=-Q(J,I)
  350 IF (I.EQ.1) GO TO 360
      IF (ABS(E(I)).LT.ABS(E(I-1))) GO TO 360
      CALL SWITCH(I,1,Q,R,E)
      N0=N0+1
  360 CONTINUE
      IF (N0.NE.0) GO TO 330
      IF (ABS(E(3)).GT.SMALL*XNRM) GO TO 370
      E(3)=0.0
      IF (ABS(E(2)).GT.SMALL*XNRM) GO TO 370
      E(2)=0.0
  370 DT=DET(Q(1,1),Q(1,2),Q(1,3))*DET(R(1,1),R(1,2),R(1,3))
C     WRITE (JOUT,501) (E(I),I=1,3)
      RETURN
  501 FORMAT (/,5X,'SINGULAR VALUES - ',1P,3E14.4)
      END
      SUBROUTINE GIVNS(A,B,M,N)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(3,3),B(3,3)
      IF (ABS(A(M,N)).LT.ABS(A(N,N))) GO TO 10
      T=A(N,N)/A(M,N)
      S=1.0/SQRT(1.0+T*T)
      C=S*T
      GO TO 20
   10 T=A(M,N)/A(N,N)
      C=1.0/SQRT(1.0+T*T)
      S=C*T
   20 DO 30 J=1,3
      V=A(M,J)
      W=A(N,J)
      X=B(J,M)
      Y=B(J,N)
      A(M,J)=C*V-S*W
      A(N,J)=S*V+C*W
      B(J,M)=C*X-S*Y
   30 B(J,N)=S*X+C*Y
      RETURN
      END
      SUBROUTINE SWITCH(N,M,U,V,D)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION U(3,3),V(3,3),D(3)
      DO 10 I=1,3
      TEM=U(I,N)
      U(I,N)=U(I,N-1)
      U(I,N-1)=TEM
      IF (M.EQ.0) GO TO 10
      TEM=V(I,N)
      V(I,N)=V(I,N-1)
      V(I,N-1)=TEM
   10 CONTINUE
      TEM=D(N)
      D(N)=D(N-1)
      D(N-1)=TEM
      RETURN
      END
      SUBROUTINE MVVAD(A,B,C,D)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(3,3),B(3),C(3),D(3)
      DO 10 J=1,3
      D(J)=C(J)
      DO 10 I=1,3
   10 D(J)=D(J)+A(J,I)*B(I)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DET (A,B,C)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(3),B(3),C(3)
      DET=A(1)*(B(2)*C(3)-B(3)*C(2))+A(2)*(B(3)*C(1)-B(1)*C(3))
     1  +A(3)*(B(1)*C(2)-B(2)*C(1))
      RETURN
      END
      SUBROUTINE MMMUL(A,B,C)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(3,3),B(3,3),C(3,3)
      DO 10 I=1,3
      DO 10 J=1,3
      C(I,J)=0.0
      DO 10 K=1,3
   10 C(I,J)=C(I,J)+A(I,K)*B(K,J)
      RETURN
      END
      SUBROUTINE KABSCH (  RMS )
C This routine is a driver to calculate rms deviations using the
C procedure of KABSCH 
C Routines  for rms calculation were originally written by  Mil Lambert.
C This module requires IMSL libraries.
      IMPLICIT REAL*8 ( A-H,O-Z)

#ifdef PHCALC
      INCLUDE 'PARAMET.ECEPPH'
#else
      INCLUDE 'PARAMET.ECEPP'
#endif
      INTEGER   I,IATOM,IATOMC2,IER

      DIMENSION  P(3),GRAD(3),WK(18)
      COMMON /REA/ REAN(3)
      INCLUDE 'COMMON.RMSFIT'
      INCLUDE 'COMMON.IOUNIT'

      IF(IATMCD.EQ.0) THEN
         WRITE(JOUT,5550) IATMCD
5550     FORMAT('ERROR  IN  SUBROUTINE KABSCH  - IATMCD = ',I7)
         RETURN
      ENDIF
C**********************DR 10/90******************************
C          WRITE(JOUT,'(A)') 'Before ORIENT1'
C          WRITE(JOUT,'(3F15.6)') (X1_AL(J,1219),J=1,3)
C          WRITE(JOUT,'(3F15.6)') (X2_AL(J,1219),J=1,3)
      CALL COM(X1,IATMCD)
      IF(NATMOM.EQ.0) THEN
         WRITE(JOUT,5551) NATMOM
5551     FORMAT('ERROR  IN  SUBROUTINE KABSCH  - NATMOM = ',I7)
         STOP  5551
      ENDIF
      DO 240 IJ=1,NATMOM
         DO 241 IK=1,3
            X1_AL(IK,IJ)=X1_AL(IK,IJ)-REAN(IK)
241      CONTINUE
240   CONTINUE

      CALL COM(X2,IATMCD)
      DO 540 IJ=1,NATMOM
         DO 541 IK=1,3
            X2_AL(IK,IJ)=X2_AL(IK,IJ)-REAN(IK)
541      CONTINUE
540   CONTINUE
C**********************DR 10/90******************************

      CALL ORIENT1(X1,X2,IATMCD,UX1)

      RMS=RMSFN(UX1,X2,IATMCD)
C     WRITE (JOUT,18) IATMCD, X1(1,1), X2(1,1)
18    FORMAT ( 'IN KABSCH 2ST CALL IATMCD - X1(1,1)-X2(1,1)',I4,2F8.3)
C     WRITE (JOUT,17) RMS
17    FORMAT ('THE RMS FROM KABSCH''S METHOD IS ',F12.8)
 200  CONTINUE
      DO 630 IJ=1,IATMCD
         DO 631 IK=1,3
            UX1(IK,IJ)=UX1(IK,IJ)+REAN(IK)
631      CONTINUE
630   CONTINUE
      DO 640 IJ=1,NATMOM
         DO 641 IK=1,3
            UX1_AL(IK,IJ)=UX1_AL(IK,IJ)+REAN(IK)
641      CONTINUE
640   CONTINUE


      RETURN
 1000     FORMAT (3F14.6)
      END


      SUBROUTINE COM(X,IATMCD)
C COM SHIFTS PEPTIDE TO CEN OF MASS
      IMPLICIT REAL*8 ( A-H,O-Z)
      INTEGER I,IATOM,IATMCD
      DIMENSION REAN(3),X(3,IATMCD)
C**********************DR 10/90******************************
C     COMMON REAT ADDED   DR. 10/90
      INCLUDE 'COMMON.IOUNIT'
      COMMON /REA/ REAT(3)
      EQUIVALENCE (REAT(1),REAN(1))
C**********************DR 10/90******************************

      DATOM=DFLOAT(IATMCD)
      DO 110 I=1,3
         REAN(I)=0.0
110   CONTINUE
      DO 120 IATOM=1,IATMCD
         DO 121 I=1,3
            REAN(I)= REAN(I)+ X(I,IATOM)
121      CONTINUE
120   CONTINUE
      DO 130 I=1,3
         REAN(I)=REAN(I)/DATOM
130   CONTINUE
      DO 140 IATOM=1,IATMCD
         DO 141 I=1,3
            X(I,IATOM)=X(I,IATOM)-REAN(I)
141      CONTINUE
140   CONTINUE
C        WRITE ( JOUT,'(3F12.6)') (REAN(I),I=1,3)
      RETURN
      END

      DOUBLE PRECISION FUNCTION RMSFN (X1,X2,IATMCD)
      IMPLICIT REAL*8 ( A-H,O-Z)
      INTEGER I,IATOM,IATMCD
      DIMENSION X1(1:3,1:IATMCD),X2(1:3,1:IATMCD)
      RMSFN=0.0
      DO 110 IATOM=1,IATMCD
        DO 111 I=1,3
           RMSFN=RMSFN+(X1(I,IATOM)-X2(I,IATOM))**2
111   CONTINUE
110   CONTINUE
      RMSFN=DSQRT(RMSFN/DFLOAT(IATMCD))
      RETURN
      END
      SUBROUTINE ERR(ERRID)
      INTEGER ERRID
      INCLUDE 'COMMON.IOUNIT'
      IF (ERRID.EQ.0) THEN
         WRITE (*,10)
         WRITE (JOUT,10)
10       FORMAT ('different number of atoms')
         STOP
      ELSEIF (ERRID.EQ.1) THEN
         WRITE (*,11)
         WRITE (JOUT,11)
11       FORMAT ('arrays dimensioned too small')
         STOP
      ELSEIF (ERRID.EQ.2) THEN
         WRITE (*,12)
         WRITE (JOUT,12)
12       FORMAT ('negative eigval')
      ELSE
         WRITE (*,13)
         WRITE (JOUT,13)
13       FORMAT ('inversion detected')
      ENDIF
      RETURN
      END
      SUBROUTINE RGYR ( INUM , RGY )

#ifdef PHCALC
      INCLUDE 'PARAMET.ECEPPH'
#else
      INCLUDE 'PARAMET.ECEPP'
#endif
      REAL*8  RR1(3), RR2, DOT, RGY
      INCLUDE 'COMMON.RMSFIT'
CD    REAL*8  X1(3,2000)
CD    REAL*8  X2(3,2000)
CD    COMMON/X1COM/ X1, IATMCD
CD    COMMON/X2COM/ X2

      RR2 = 0.0
      DO 20 J=1,3
        RR1(J) = 0.0
 20   CONTINUE

      IF (INUM.GT.0) THEN
        DO 40 I=1,IATMCD
          DO 30 J=1,3
            RR1(J) = RR1(J) +  X1 (J,I)
            RR2 = RR2 + X1(J,I)**2
  30      CONTINUE
  40    CONTINUE
      ELSE
        DO 60 I=1,IATMCD
          DO 50 J=1,3
            RR1(J) = RR1(J) +  X2 (J,I)
            RR2 = RR2 + X2(J,I)**2
  50      CONTINUE
  60    CONTINUE
      END IF
      DOT = 0.0
      DO 80 J=1,3
        DOT = DOT + RR1(J)**2
  80  CONTINUE

      RGY = (RR2 - DOT/FLOAT(IATMCD)) /FLOAT(IATMCD)
      RGY = DSQRT (RGY )
      RETURN
 1000 FORMAT ( 'CONF #, R GYR ', I10, F12.6)
      END
