c SIMS: public
c Smooth Invariant Molecular Surface
c Yury N Vorobjev
c All copyright reserved
c to obtain copy of SIMS contact
c (919)966-8625; Fax: (919)966-2852
c vorobjev@femto.med.unc.edu



        real*8 FUNCTION DDIST(A,B)


        implicit none

        REAL*8 A(3)

        REAL*8 B(3)

        DDIST = DSQRT((A(1)-B(1))**2 + (A(2)-B(2))**2 + (A(3)-B(3))**2)

        RETURN

        END


        REAL*8 FUNCTION DDIST2(A,B)
        
        implicit none

        REAL*8 A(3)

        REAL*8 B(3)

        DDIST2 = (A(1)-B(1))**2 + (A(2)-B(2))**2 + (A(3)-B(3))**2

        RETURN

        END


        REAL*8 FUNCTION DDLENG(A)

        implicit none

        REAL*8 A(3)

        DDLENG = DSQRT(A(1)**2 + A(2)**2 + A(3)**2)

        RETURN

        END



        SUBROUTINE DVECTPR(A,B,C)


        implicit none

        REAL*8 A(3)

        REAL*8 B(3)

        REAL*8 C(3)

        C(1) = A(2) * B(3) - A(3) * B(2)

        C(2) = A(3) * B(1) - A(1) * B(3)

        C(3) = A(1) * B(2) - A(2) * B(1)

        RETURN

        END



        SUBROUTINE DVNORM(A,B)


        implicit none

        REAL*8 A(3),B(3)
	real*8 V
	integer K

        V = DSQRT(A(1)**2 + A(2)**2 + A(3)**2)

        DO  K = 1,3
           B(K) = A(K) / V
        END DO !K

        RETURN

        END


        SUBROUTINE VPERP(A,B)

	implicit none

        REAL*8 A(3)

        REAL*8 B(3)

        REAL*8 P(3)

	real*8 SMALL,DT
	integer M,K


        SMALL = 10000.0d0

        M = 0

        DO K = 1,3
           IF (DABS(A(K)) .GE. SMALL) GO TO 50
           SMALL = DABS(A(K))
           M = K
50       end do !K

        DO K = 1,3
           B(K) = 0.0d0
           IF (K .EQ. M) B(K) = 1.0d0
         end do !K


        DT = A(M) / (A(1)**2 + A(2)**2 + A(3)**2)

        DO K = 1, 3
           P(K) = DT * A(K)

           B(K) = B(K) - P(K)

        end do !K


        CALL DVNORM(B,B)

        RETURN

        END




        SUBROUTINE CONJ(H,G,GHGT)


        implicit none
        REAL*8 G(3,3)

        REAL*8 H(3,3)

        REAL*8 GHGT(3,3)

        REAL*8 GT(3,3)

	integer K,L



        CALL IMATX(GHGT)

        CALL CAT(GHGT,G)

        CALL CAT(GHGT,H)


        DO K = 1,3

           DO L = 1,3

              GT(K,L) = G(L,K)

           end do !L
         end do!K

        CALL CAT(GHGT,GT)

        RETURN

        END


        SUBROUTINE IMATX(A)

        implicit none

        REAL*8 A(3,3)
	integer I,J

        DO I = 1,3

           DO J = 1,3

              A(I,J) = 0.0d0

           end do !J

           A(I,I) = 1.0d0

        end do !I

        RETURN

        END


C       real*8 FUNCTION DET(A,B,C)


C       implicit none

C       real*8 DDOT

C       REAL*8 A(3)

C       REAL*8 B(3)

C       REAL*8 C(3)

C       REAL*8 AB(3)

C       CALL DVECTPR(A,B,AB)

C       DET = DDOT(AB,C)

C       RETURN

C       END





        LOGICAL FUNCTION COLLID(P,RP,CNBR,ERNBR,MNBR,NNBR,MAXNBR,ISHAPE,

     1   JNBR,KNBR,MOLNBR,IMOL,LKF,LKNBR)


 
c COLLID = .TRUE.     if there is COLLISION
	
	implicit none

	integer*4 NNBR,JNBR,KNBR,MAXNBR,LKF
	
	real*8 toler

        REAL*8 P(3),RP

        REAL*8 CNBR(3,MAXNBR)

        REAL*8 ERNBR(MAXNBR)

        LOGICAL*1 MNBR(MAXNBR)

        INTEGER*2 MOLNBR(MAXNBR)

        INTEGER*2 LKNBR(MAXNBR)

        INTEGER*2 IMOL,ISHAPE

	integer I
	real*8 VECT1,VECT2,VECT3,SR2,DD2
	 real*8  toler_nb,toler_pr,toler_yon,toler_d,toler_cx,toler_ovr 
	 real*8  ktoler_yon
	 common/TOLERANCE/toler_nb,toler_pr,toler_yon,toler_d,
     &	 toler_cx,toler_ovr,ktoler_yon


c atoms collided if DIJ< Ri + RJ +toler
	
	toler= toler_nb    ! toler for nearest neighb determination

        I = LKF

        GO TO 100

50      CONTINUE

        I = LKNBR(I)

100     CONTINUE

        IF (I .EQ. 0) GO TO 150

        VECT1 = DABS(P(1) - CNBR(1,I)) + toler

        IF (VECT1 .GE. ERNBR(I)) GO TO 50

        VECT2 = DABS(P(2) - CNBR(2,I)) + toler

        IF (VECT2 .GE. ERNBR(I)) GO TO 50

        VECT3 = DABS(P(3) - CNBR(3,I)) + toler

        IF (VECT3 .GE. ERNBR(I)) GO TO 50

        IF (I .EQ. JNBR .OR. I .EQ. KNBR) GO TO 50

        SR2 = ERNBR(I) ** 2

        DD2 = VECT1 ** 2 + VECT2 ** 2 + VECT3 ** 2

        IF (DD2 .GE. SR2) GO TO 50

        COLLID = .TRUE.

        RETURN

150     CONTINUE

        COLLID = .FALSE.

        RETURN

        END


        LOGICAL FUNCTION BURIED(P,RP,CNBR,RNBR,MNBR,NNBR,MAXNBR,ISHAPE,

     1   JNBR,KNBR,MOLNBR,IMOL)



	implicit none
	
	integer*4 NNBR,JNBR,KNBR,MAXNBR

        REAL*8 P(3),RP

        REAL*8 CNBR(3,MAXNBR)

        REAL*8 RNBR(MAXNBR)

        LOGICAL*1 MNBR(MAXNBR)

        INTEGER*2 MOLNBR(MAXNBR)

        INTEGER*2 IMOL,ISHAPE
	integer I
	real*8 VECT1,VECT2,VECT3,SR2,DD2,SUMRAD


        IF (NNBR .LE. 0) GO TO 100


        DO 50 I = 1, NNBR

           IF (IMOL .EQ. MOLNBR(I)) GO TO 50

           IF (ISHAPE .GT. 1 .AND. I .EQ. JNBR) GO TO 50

           IF (ISHAPE .EQ. 3 .AND. (I .EQ. KNBR .OR. .NOT. MNBR(I)))

     1   GO TO 50

           SUMRAD = RP + RNBR(I)

           VECT1 = DABS(P(1) - CNBR(1,I))

           IF (VECT1 .GE. SUMRAD) GO TO 50

           VECT2 = DABS(P(2) - CNBR(2,I))

           IF (VECT2 .GE. SUMRAD) GO TO 50

           VECT3 = DABS(P(3) - CNBR(3,I))

           IF (VECT3 .GE. SUMRAD) GO TO 50

           SR2 = SUMRAD ** 2

           DD2 = VECT1 ** 2 + VECT2 ** 2 + VECT3 ** 2

           IF (DD2 .LT. SR2) GO TO 150

50      CONTINUE

100     CONTINUE

        BURIED = .FALSE.

        GO TO 200

150     CONTINUE

        BURIED = .TRUE.

200     CONTINUE

        RETURN

        END


        SUBROUTINE GENUN(U,AR,N)


	implicit none

	integer N

        REAL*8 U(3,N)
	real*8 AR(N)

	real*8 PI,FI,FJ,Z,XY,X,Y
	real*8 twoPI
	real*8 dNHOR,dNVERT
	real*8 aat,aaf
	real*8 area
	real*8 dtet,dtet2,sdtet,cdtet

	integer NEQUAT,NVERT,I,NHOR,J,NU

       real*8 PI2,null,onehalf,one,two,three,four,big,small
       common/CONSTANTI/PI2,null,onehalf,one,two,three,four,big,small

	PI = PI2     ! 3.1415927d0
	twoPI=two*PI

	area = null 

        NEQUAT = DSQRT(dfloat(N ) * PI)

        NVERT = onehalf * NEQUAT

        IF (NVERT .LT. 1) NVERT = 1

        NU = 0

	dNVERT=dfloat(NVERT)

	dtet = PI/dNVERT
	dtet2= dtet*onehalf
	sdtet =two* dsin(dtet2)
	cdtet = dcos(dtet2)

        DO 100 I = 0,NVERT

	   FI = dtet*I

           Z = DCOS(FI)

           XY = DSIN(FI)
           
	   if(I.eq.0.or.I.eq.NVERT)then
	   aat=1.0d0 - cdtet
	   else
	   aat=XY*sdtet
	   end if

           NHOR = NEQUAT * XY

           IF (NHOR .LT. 1) NHOR = 1

	   dNHOR=dfloat(NHOR)

	   aaf=twoPI/dNHOR

           DO 50 J = 0,NHOR-1

              FJ = aaf*J

              X = DCOS(FJ) * XY

              Y = DSIN(FJ) * XY

              IF (NU .GE. N) GO TO 150

              NU = NU + 1

              U(1,NU) = X

              U(2,NU) = Y

              U(3,NU) = Z

	      AR(NU) = aaf*aat

	      area=area + AR(NU)

50         CONTINUE

100     CONTINUE

150     CONTINUE

        N = NU

        RETURN

        END
c 
        SUBROUTINE GENUN01(RI,U,AR,TET,N)


	implicit none

	integer N
        
	real*8 RI
        REAL*8 U(3,N)
	real*8 AR(N),TET(N)

	real*8 PI,FI,FJ,Z,XY,X,Y
	real*8 twoPI
	real*8 dNHOR,dNVERT
	real*8 aat,aaf,RI2
	real*8 area
	real*8 dtet,dtet2,sdtet,cdtet

	integer NEQUAT,NVERT,I,NHOR,J,NU
	integer NHORmin
	logical CONTROL

       real*8 PI2,null,onehalf,one,two,three,four,big,small
       common/CONSTANTI/PI2,null,onehalf,one,two,three,four,big,small

        data NHORmin/3/
	
	CONTROL = .false.

	PI = PI2     ! 3.1415927d0
	twoPI=two*PI
	RI2 = RI*RI

	area = null 

        NEQUAT = DSQRT(dfloat(N ) * PI)

        NVERT = onehalf * NEQUAT

        IF (NVERT .LT. 1) NVERT = 1

        NU = 0

	dNVERT=dfloat(NVERT)

	dtet = PI/dNVERT
	dtet2= dtet*onehalf
	sdtet =two* dsin(dtet2)
	cdtet = dcos(dtet2)

        DO 100 I = 0,NVERT

	   FI = dtet*I

           Z = RI*DCOS(FI)

           XY = DSIN(FI)
           
	   if(I.eq.0.or.I.eq.NVERT)then
	   aat=1.0d0 - cdtet
	   else
	   aat=XY*sdtet
	   end if

           NHOR = NEQUAT * XY

	   IF (NHOR .LT. NHORmin) NHOR = NHORmin
           if(I.eq.0.or.I.eq.NVERT) NHOR = 1

	   dNHOR=dfloat(NHOR)

	   aaf=twoPI/dNHOR

	   XY = XY*RI

           DO 50 J = 0,NHOR-1

              FJ = aaf*J

              X = DCOS(FJ) * XY

              Y = DSIN(FJ) * XY

              IF (NU .GE. N) GO TO 150

              NU = NU + 1

              U(1,NU) = X

              U(2,NU) = Y

              U(3,NU) = Z

	      AR(NU) = aaf*aat*RI2

	      TET(NU) = FI

	      area=area + AR(NU)

50         CONTINUE

100     CONTINUE

150     CONTINUE

        N = NU

        aaf = 4.0d0*PI*RI2/area

	if(CONTROL)then
	write(*,'(a12,i5,a12,f16.14)')
     &	'GENUN01:N =',NU,' area norm=',aaf
	end if

	do i=1,NU
	AR(i) = aaf*AR(i)
	end do

        RETURN

        END

c
        SUBROUTINE GENUN02(RI,U,AR,NCHI,N,NC)


	implicit none
       
	
	integer N,NC

	REAL*8 RI                  !radius of the shpere
        REAL*8 U(3,NC,N)  
	real*8 AR(N)
	integer NCHI(N)

	real*8 PI,FI,FJ,Z,XY,X,Y
	real*8 twoPI
	real*8 dNHOR,dNVERT
	real*8 aat,aaf
	real*8 area,RI2
	real*8 dtet,dtet2,sdtet,cdtet

       real*8 dfich9(9),dtetch9(9)    ! delta fi and delta tet for chilren dots 
       real*8 dfich09(9),dtetch09(9)
       real*8 dfich5(5),dtetch5(5)    ! delta fi and delta tet for chilren dots 
       real*8 dfich05(5),dtetch05(5)
       real*8 dfich(9),dtetch(9)    ! delta fi and delta tet for chilren dots 
       real*8 dfich0(9),dtetch0(9)
       real*8 chstep9,chstep5
       integer ncloc
       real*8 chstep,chsfi
       real*8 chstet,chstet1,chstet2
       real*8 aa,bb,tetch,fich
       real*8 aafc

       integer NEQUAT,NVERT,I,NHOR,J,NU
       integer NHORmin
       integer ich,ich1,k,k1,k2
       integer print,iprint0,iprint1
	
	logical CONTROL

       real*8 PI2,null,onehalf,one,two,three,four,big,small
       common/CONSTANTI/PI2,null,onehalf,one,two,three,four,big,small
c
       data dfich9/0.0d0,1.0d0,0.0d0,-1.0d0,-1.0d0,-1.0d0,
     &       0.0d0,1.0d0,1.0d0/

       data dtetch9/0.0d0,-1.0d0,-1.0d0,-1.0d0,0.0d0,1.0d0,
     & 1.0d0,1.0d0,0.0d0/

       data dtetch09/0.0d0,1.0d0,1.0d0,1.0d0,1.0d0,1.0d0,
     & 1.0d0,1.0d0,1.0d0/

       data dfich09/0.0d0,0.0d0,1.0d0,2.0d0,3.0d0,4.0d0,5.0d0,
     & 6.0d0,7.0d0/

       data chstep9/0.33333d0/

c
       data dfich5/0.0d0,1.0d0,-1.0d0,-1.0d0,1.0d0/
       data dtetch5/0.0d0,-1.0d0,-1.0d0,1.0d0,1.0d0/
       data dtetch05/0.0d0,1.0d0,1.0d0,1.0d0,1.0d0/
       data dfich05/0.0d0,1.0d0,3.0d0,5.0d0,7.0d0/
       data chstep5/0.250d0/
       
       data NHORmin/3/

       iprint0 = 0 
       iprint1 = 1
       print  =  -1
	
       CONTROL = .false. 

	if(print.ge.iprint0)then
	write(*,*)'GENUN02 start:'
	end if
c
c ASSIGN data
c
	ncloc = NC
	if(NC.eq.9)then
	do k=1,ncloc
        dfich(k)=dfich9(k)
	dtetch(k)=dtetch9(k)
	dfich0(k)=dfich09(k)
	dtetch0(k)=dtetch09(k)
	end do !k
	chstep = chstep9
	end if! 9

	if(NC.eq.5)then
	do k=1,ncloc
        dfich(k)=dfich5(k)
	dtetch(k)=dtetch5(k)
	dfich0(k)=dfich05(k)
	dtetch0(k)=dtetch05(k)
	end do !k
	chstep = chstep5
	end if! 5
c
	PI = PI2     ! 3.1415927d0
        twoPI=two*PI

	RI2 = RI**2

	area = null 

        NEQUAT = DSQRT(dfloat(N ) * PI)

        NVERT = onehalf * NEQUAT

        IF (NVERT .LT. 1) NVERT = 1

        NU = 0

	dNVERT=dfloat(NVERT)

	dtet = PI/dNVERT
	dtet2= dtet*onehalf
	chstet1 = chstep*dtet
	chstet2 = chstet1*two

	sdtet =two* dsin(dtet2)
	cdtet = dcos(dtet2)

        DO 100 I = 0,NVERT

	   FI = dtet*I

           Z = RI*DCOS(FI)

           XY = DSIN(FI)
           
	   if(I.eq.0.or.I.eq.NVERT)then !area of element
	   aat=1.0d0 - cdtet
	   else
	   aat=XY*sdtet
	   end if

           NHOR = NEQUAT * XY

           IF (NHOR .LT. NHORmin) NHOR = NHORmin
	   if(I.eq.0.or.I.eq.NVERT) NHOR = 1

	   dNHOR=dfloat(NHOR)

	   aaf=twoPI/dNHOR

	   if(NHOR.eq.1)then
	   chsfi = twoPI*0.125d0 !Pole parent dot for 9 and 5 children
	   chstet = dtet*0.66667d0
           else
	   chsfi = aaf*chstep
	   chstet = chstet1
	   end if

	   XY = XY*RI

           DO 50 J = 0,NHOR-1

              FJ = aaf*J

              X = DCOS(FJ) * XY

              Y = DSIN(FJ) * XY

              IF (NU .GE. N)then
	      write(*,*)'GENUN02:ERROR: sphera is not completed'
	      GO TO 150
	      end if

              NU = NU + 1

	      NCHI(NU) = 1    ! children dot number (parent = 1 children)

              U(1,1,NU) = X

              U(2,1,NU) = Y

              U(3,1,NU) = Z

	      AR(NU) = RI2*aaf*aat

	      area = area + AR(NU)

	      if(ncloc.ge.5)then  !4, 8 - child dots
	      do ich = 2,ncloc        ! central child=1 = parent
	      if(NHOR.eq.1)then ! Pole parent dot
	      if(I.eq.0) tetch = FI + dtetch0(ich)*chstet       !N pole
	      if(I.eq.NVERT) tetch = FI - dtetch0(ich)*chstet   !S pole

	      fich  = dfich0(ich)*chsfi
	      aa = RI*dsin(tetch)
	      else  
	      tetch = FI + dtetch(ich)*chstet
              aa = RI*dsin(tetch)

	      if(XY.gt.small)then
	      aafc=aa/XY
	      else
	      aafc=one
	      end if

	      fich  = FJ + dfich(ich)*chsfi*aafc
	      end if

	      U(1,ich,NU)=aa*dcos(fich)
	      U(2,ich,NU)=aa*dsin(fich)
	      U(3,ich,NU)=RI*dcos(tetch)

	      NCHI(NU)=NCHI(NU) + 1 ! number of children dots for NU parent
	      end do!ich
	      end if


50      CONTINUE

100     CONTINUE

150     CONTINUE

        N = NU

        aaf = 4.0d0*PI*RI2/area
	if(CONTROL)then
	write(*,'(a12,i5,a12,f16.14)')
     &	'GENUN02:N =',NU,' area norm=',aaf
	end if

	do i=1,NU
	AR(i) = aaf*AR(i)
	end do

	if(print.ge.iprint1)then
	write(*,*)'coordinates'
	do k=1,N
	write(*,'(a10,i5,3f10.6)')'parent ',k,(U(k1,1,k),k1=1,3)
	do k2=2,ncloc
	write(*,'(a10,i5,3f10.6)')'childr ',k2,(U(k1,k2,k),k1=1,3)
	end do
	end do
	end if
	if(print.ge.iprint0)then
	write(*,*)'GENUN02 finish: Nvect', N
	end if

        RETURN
        END





        SUBROUTINE ERROR(NUMBER,INT,FLOAT)

#ifdef PHCALC
      INCLUDE 'PARAMET.ECEPPH'
#else
      INCLUDE 'PARAMET.ECEPP'
#endif
        include'param.ibems'
 
C       include'surf-sims.h'

        integer LIST(15)
        integer I,NUMBER
        integer INT
        real*8 FLOAT

        DATA LIST/120,127,130,140,150,160,170,

     1   210,320,440,480,720,760,830,850/


        DO 50 I = 1,15

           IF (LIST(I) .EQ. NUMBER) GO TO 150

50      CONTINUE

        WRITE (*,100)
        WRITE (kanalp,100)

100     FORMAT(1X,'ERROR OF UNIDENTIFIABLE TYPE')

        STOP  4001

150     CONTINUE


        GO TO (200,300,400,500,600,700,800,900,1000,1100,1200,

     1   1300,1400,1500,1600) I


200     WRITE (kanalp,250) NUMBER,FLOAT

250     FORMAT(1X,'ERROR',I5,2X,'NEGATIVE PROBE RADIUS:',F10.5)

        STOP  4002

300     WRITE (kanalp,350) NUMBER,INT

350     FORMAT(1X,'ERROR',I5,2X,'BAD BURIED SURFACE FLAG:',I5)

        STOP  4003

400     WRITE (kanalp,450) NUMBER,INT

450     FORMAT(1X,'ERROR',I5,2X,'TOO FEW OR TOO MANY ATOM TYPES:',I5)

        STOP  4004

500     WRITE (kanalp,550) NUMBER,FLOAT,INT

550     FORMAT(1X,'ERROR',I5,2X,'NEGATIVE ATOM RADIUS:',

     1   F10.5,' ATOM',I5)

        STOP  4005

600     WRITE (kanalp,650) NUMBER,INT

650     FORMAT(1X,'ERROR',I5,2X,'TOO MANY ATOMS:',I5)

        STOP  4006

700     WRITE (kanalp,750) NUMBER,INT

750     FORMAT(1X,'ERROR',I5,2X,

     1   'INVALID SURFACE REQUEST NUMBER FOR ATOM:',I5)

        STOP  4007

800     WRITE (kanalp,850) NUMBER,INT

850     FORMAT(1X,'ERROR',I5,2X,'INVALID ATOM TYPE FOR ATOM:',I5)

        STOP  4008

900     WRITE (kanalp,950) NUMBER,INT

950     FORMAT(1X,'ERROR',I5,2X,'TOO MANY NEIGHBORS:',I5)

        STOP  4009

1000    WRITE (kanalp,1050) NUMBER,INT

1050    FORMAT(1X,'ERROR',I5,2X,'TOO MANY POINTS FOR REENTRANT PROBE:',

     1  I5)

        STOP  4010

1100    WRITE (kanalp,1150) NUMBER,INT

1150    FORMAT(1X,'ERROR',I5,2X,'TOO MANY POINTS FOR ARC:',I5)

        STOP  4011

1200    WRITE (kanalp,1250) NUMBER,INT

1250    FORMAT(1X,'ERROR',I5,2X,'TOO MANY POINTS FOR REENTRANT PROBE:',

     1  I5)

        STOP  4012

1300    WRITE (kanalp,1350) NUMBER,INT

1350    FORMAT(1X,'ERROR',I5,2X,'TOO MANY YON PROBES:',I5)

        STOP  4013

1400    WRITE (kanalp,1450) NUMBER,INT

1450    FORMAT(1X,'ERROR',I5,2X,'TOO MANY VICTIM PROBES:',I5)

        STOP  4014

1500    WRITE (kanalp,1550) NUMBER,INT

1550    FORMAT(1X,'ERROR',I5,2X,'TOO MANY EATERS:',I5)

        STOP  4015

1600    WRITE (kanalp,1650) NUMBER,INT

1650    FORMAT(1X,'ERROR',I5,2X,'TOO MANY EATERS:',I5)

        STOP  4016

        END

c------------------------------------------------------------------
c          ms_M:  subroutines
c
c------------------------------------------------------------------
	logical function compare_ij(i,j,n,m,l)

	implicit none
	integer i,j,n,m,l
        
	compare_ij=.false.

	if(.not.(i.eq.n.or.i.eq.m.or.i.eq.l))then
	return
	end if
	if(.not.(j.eq.n.or.j.eq.m.or.j.eq.l))then
	return
	end if

	compare_ij=.true.
	return
	end

	logical function compare_ijk(i,j,k,n,m,l)

	implicit none
	integer i,j,k,n,m,l
        
	compare_ijk=.false.

	if(.not.(n.eq.i.or.n.eq.j.or.n.eq.k))then
	return
	end if
	if(.not.(m.eq.i.or.m.eq.j.or.m.eq.k))then
	return
	end if
	if(.not.(l.eq.i.or.l.eq.j.or.l.eq.k))then
	return
	end if

	compare_ijk=.true.
	return
	end

c
        real*8 function triang_area(a,b,c)
        implicit none
        real*8 a(3),b(3),c(3)
        real*8 aa,bb,ab,ak,bk,ar
        integer k
        aa=0.0D0
        bb=0.0D0
        ab=0.0D0
        do k=1,3
          ak=b(k)-a(k)
          bk=c(k)-a(k)
          aa=aa+ak*ak
          bb=bb+bk*bk
          ab=ab+ak*bk
        enddo

	triang_area=0.0d0

	ar=aa*bb-ab*ab
	if(ar.ge.0.0d0)triang_area=0.5D0*dsqrt(ar)
        return
        end
c------------------------------------------------------------------------
        real*8 function curv_triang_area(a,b,c,center,r)
c
c
        implicit none
        real*8 a(3),b(3),c(3)
	real*8 center(3),r
        real*8 tol /1.0D-4/, one /1.0D0/
        real*8 triang_area
        real*8 aa,bb,cc,ak,bk,cost12,cost13,cost23,r2,
     &   sint12,sint13,sint23,cosa,cosb,cosg,sina,sinb,sing,cossurf
        integer k

c       print *,'a:',a
c       print *,'b:',b
c       print *,'c:',c

        cost12=0.0D0
        cost13=0.0D0
        cost23=0.0D0
        r2=r*r
        do k=1,3
          aa=a(k)-center(k)
          bb=b(k)-center(k)
          cc=c(k)-center(k)
          cost12=cost12+aa*bb
          cost13=cost13+aa*cc
          cost23=cost23+bb*cc
        enddo
        cost12=cost12/r2   
        cost13=cost13/r2   
        cost23=cost23/r2
        sint12=1.0D0-cost12*cost12
        sint13=1.0D0-cost13*cost13
        sint23=1.0D0-cost23*cost23
        if (sint12.lt.tol .or. sint13.lt.tol .or. sint23.lt.tol) then
          curv_triang_area=triang_area(a,b,c)
          return
        endif
        sint12=dsqrt(sint12)
        sint13=dsqrt(sint13)
        sint23=dsqrt(sint23)
c        print *,'cost12=',cost12,' cost13=',cost13,' cost23=',cost23
        cosa=(cost23-cost12*cost13)/(sint12*sint13)
        cosb=(cost13-cost12*cost23)/(sint12*sint23)
        cosg=(cost12-cost13*cost23)/(sint13*sint23)
        sina=1.0D0-cosa*cosa
        sinb=1.0D0-cosb*cosb
        sing=1.0D0-cosg*cosg
        if (sina.lt.tol .or. sinb.lt.tol .or. sing.lt.tol) then
          curv_triang_area=triang_area(a,b,c)
          return   
        endif
        sina=dsqrt(sina)
        sinb=dsqrt(sinb)
        sing=dsqrt(sing)

c        print *,'cosa=',cosa,' cosb=',cosb,' cosg=',cosg
c        print *,'cos(area/R**2)=',-cosa*cosb*cosg+sina*sinb*cosg
c     &                          +sina*cosb*sing+cosa*sinb*sing

        cossurf=-cosa*cosb*cosg+sina*sinb*cosg
     &                          +sina*cosb*sing+cosa*sinb*sing
        if (dabs(cossurf).gt.1.0D0) then 
          curv_triang_area=0.0D0
        else
          curv_triang_area=r2*dacos(cossurf)
        endif
        return
        end

c

	 subroutine gener_conc_dot01(v1,v2,v3,rpr,dens,
     &	 area_cc,vdot,area_dot,vndot,ndot)

         implicit none
         integer ndotccmx
         parameter(ndotccmx=128)
         integer ndot
         real*8 v1(3),v2(3),v3(3),rpr,dens
         real*8 vdot(3,ndotccmx),area_dot(ndotccmx),vndot(3,ndotccmx)

         real*8 area_cc,center(3)
	 real*8 aa,bb,cc,r2,empk,aa1
         real*8 cost(3),tedgemx,costm,dens_l
c real function:
         real*8 curv_triang_area

	 logical control

c tesselation data
         integer tmax,i,k,j,klev,nd,klevmax,nd_l
         parameter(tmax=4)
         integer ntriangl(tmax)
         data ntriangl/1,4,16,64/
	 data center/0.0d0,0.0d0,0.0d0/

	 control = .false.

         klevmax=4
c  total area
	 area_cc=curv_triang_area(v1,v2,v3,center,rpr)

c  estimated number of dots
	nd= dint(area_cc*dens + 0.5d0)    ! area controlled

c  estimate number of dots : max lenght of triangle edges
        empk = 2.00d0
        r2=rpr*rpr
        do k=1,3
           cost(k)=0.0d0
        end do !k

        do k=1,3
          aa=v1(k)-center(k)
          bb=v2(k)-center(k)
          cc=v3(k)-center(k)
          cost(1)=cost(1)+aa*bb
          cost(2)=cost(2)+aa*cc
          cost(3)=cost(3)+bb*cc
        enddo
        
        costm=1.0d6     
        do k=1,3
        if(costm.gt.cost(k))costm=cost(k)
        end do !k
         
        tedgemx=rpr*dacos(costm/r2)       !max edge of the triangle
        dens_l = empk*dsqrt(1.25d0/dens) !average edge length of small triangle
        nd_l = dint(tedgemx/dens_l)
        if(nd_l.lt.1)nd_l=1
        nd_l = 4**(nd_l - 1)             !number of triangles

c take maximal value 
        if(nd_l.gt.nd)nd=nd_l     ! final number of triangles

	
	klev = 1
	if(nd.le.ntriangl(2)/2)klev=1
	if(nd.gt.ntriangl(2)/2.and.nd.le.ntriangl(3)/2)klev=2
	if(nd.gt.ntriangl(3)/2.and.nd.le.ntriangl(4)/2)klev=3
	if(nd.gt.ntriangl(4)/2)klev=4
        if(klev.gt.klevmax)klev=klevmax


c generate children vertexes  	 
c          dot number
c          dot vectors on probe sphere
c          dot normals
c          dot areas
        if(klev.eq.1)then   	
	call gener_cc1(v1,v2,v3,rpr,area_cc,ndot,vdot,area_dot,vndot)
        goto 1010
	end if

	if(klev.eq.2)then
	call gener_cc2(v1,v2,v3,rpr,area_cc,ndot,vdot,area_dot,vndot)
	goto 1010
	end if

	if(klev.eq.3)then
       call gener_cc3(v1,v2,v3,rpr,area_cc,ndot,vdot,area_dot,vndot)
       goto 1010
       end if

       if(klev.eq.4)then
       call gener_cc4(v1,v2,v3,rpr,area_cc,ndot,vdot,area_dot,vndot)
       goto 1010
       end if

1010   continue

       if(control)then
       aa=0.0d0
       bb=0.0d0
       aa1=area_cc/dfloat(ndot)
       do i=1,ndot
       aa=aa + area_dot(i)
       bb=bb + (area_dot(i)-aa1)**2
       end do
       bb=dsqrt(bb/dfloat(ndot))
       bb=bb/aa1
       write(*,*)'gener_cc_dot:ndot=',ndot
       write(*,*)'Area RMS for Dots bb(relative)=',bb
       write(*,*)'AREA_CC control: area_cc, sum=',area_cc,aa
       end if !control
			      
c new level can be added
       return
       end

c 
       subroutine gener_cc1(v1,v2,v3,rpr,area,ndot,vdot,area_dot,vndot)

       implicit none
         integer ndotccmx
         parameter(ndotccmx=128)
         integer ndot
         real*8 v1(3),v2(3),v3(3),rpr,area
         real*8 vdot(3,ndotccmx),area_dot(ndotccmx),vndot(3,ndotccmx)

       real*8 s,rs
       integer i,k

       ndot=1
       s=0.0d0
       do k=1,3
       vdot(k,ndot)=v1(k)+v2(k)+v3(k)
       s=s + vdot(k,ndot)**2
       end do!k

       s=dsqrt(s)
       s=1.0d0/s
       rs=rpr*s

       do k=1,3
       vdot(k,ndot)=vdot(k,ndot)*rs
       vndot(k,ndot)=vdot(k,ndot)*s
       end do !k

       area_dot(ndot)=area

       return
       end
c
       subroutine gener_cc2(v1,v2,v3,rpr,area,ndot,vdot,area_dot,vndot)


       implicit none
         integer ndotccmx
         parameter(ndotccmx=128)
         integer ndot
         real*8 v1(3),v2(3),v3(3),rpr,area
         real*8 vdot(3,ndotccmx),area_dot(ndotccmx),vndot(3,ndotccmx)

c 
       integer ntrimx
       parameter (ntrimx=4)
       integer nvertmx
       parameter (nvertmx=3+3)

c real function:
       real*8 curv_triang_area

       real*8 one3,s,rs,ar,vertex(3,nvertmx)
       real*8 center(3)
       integer vertex_parent(2,nvertmx)
       integer tri_parent(3,ntrimx)
       integer i,j,k,l
       logical OPT_AreaEachDot,OPT_norm
       data vertex_parent/1,1, 2,2, 3,3, 1,2, 2,3, 1,3/ ! vert 1,2,3,4,5,6
       data tri_parent/4,5,6, 1,4,6, 3,5,6, 2,4,5/
       data center/0.0d0,0.0d0,0.0d0/

       OPT_AreaEachDot=.true. 
       OPT_norm = .true. 

       ndot=ntrimx
       ar=area/dfloat(ndot)
       one3=1.0d0/3.0d0

       do l=1,3
       vertex(l,1)=v1(l)
       vertex(l,2)=v2(l)
       vertex(l,3)=v3(l)
       end do !l

       do i=4,nvertmx
       s=0.0d0
       do l=1,3
       vertex(l,i)=(vertex(l,vertex_parent(1,i))+
     &              vertex(l,vertex_parent(2,i)))*0.5d0
       if(OPT_norm)then
       s=s+vertex(l,i)**2
       end if
       end do!l
       if(OPT_norm)then
       rs=rpr/dsqrt(s)
       do l=1,3
       vertex(l,i)=vertex(l,i)*rs
       end do!l
       end if
       end do!i

       do i=1,ntrimx
       s=0.0d0
       do k=1,3
       vdot(k,i)=vertex(k,tri_parent(1,i))+
     &           vertex(k,tri_parent(2,i))+
     &           vertex(k,tri_parent(3,i))
       s=s+vdot(k,i)**2
       end do!k

       s=1.0d0/dsqrt(s)
       rs=rpr*s

       do k=1,3
       vndot(k,i)=vdot(k,i)*s
       vdot(k,i)=vdot(k,i)*rs
       end do!k

       if(.not.OPT_AreaEachDot)then
       area_dot(i)=ar
       else

      area_dot(i)=curv_triang_area(vertex(1,tri_parent(1,i)),
     &	                         vertex(1,tri_parent(2,i)),
     &                           vertex(1,tri_parent(3,i)),
     &                           center,rpr)

       end if

       end do!i

c 
       return
       end
c

       subroutine gener_cc3(v1,v2,v3,rpr,area,ndot,vdot,area_dot,vndot)


       implicit none
         integer ndotccmx
         parameter(ndotccmx=128)
         integer ndot
         real*8 v1(3),v2(3),v3(3),rpr,area
         real*8 vdot(3,ndotccmx),area_dot(ndotccmx),vndot(3,ndotccmx)
c real function:
       real*8 curv_triang_area
c 
       integer ntrimx        !number of dots-triangles
       parameter (ntrimx=16)
       integer nvertmx
       parameter (nvertmx=3+12) !number of vertexes

       real*8 one3,s,rs,ar,vertex(3,nvertmx)
       real*8 center(3)
       integer vertex_parent(2,nvertmx)
       integer tri_parent(3,ntrimx)
       integer nvert1,nvert2,nvert3
       integer i,j,k,l
       logical OPT_AreaEachDot,OPT_norm

       data center/0.0d0,0.0d0,0.0d0/
       data vertex_parent/1,1, 2,2, 3,3,  ! lev1  vert 1,2,3
     & 1,2, 2,3, 1,3,                     ! lev2 - vert 4,5,6
     & 1,4, 1,6, 4,6, 4,5, 5,6, 3,5, 3,6, 2,4, 2,5/ 

       data tri_parent/9,10,11, 6,9,11, 4,9,10, 5,10,11, 
     & 1,7,8, 7,8,9, 6,8,9, 4,7,9,
     & 3,12,13, 11,12,13, 5,11,12, 6,11,13,
     & 10,14,15, 4,10,14, 5,10,15, 2,14,15 / 
       
       OPT_AreaEachDot=.true. 
       OPT_norm = .true. 

       ndot=ntrimx
       ar=area/dfloat(ndot)
       one3=1.0d0/3.0d0

c generate all vertex
       nvert1=3
       do l=1,3
       vertex(l,1)=v1(l)
       vertex(l,2)=v2(l)
       vertex(l,3)=v3(l)
       end do !l

c NOnorm
       nvert2=6
       do i=nvert1+1,nvert2
       s=0.0d0
       do l=1,3
       vertex(l,i)=(vertex(l,vertex_parent(1,i))+
     &              vertex(l,vertex_parent(2,i)))*0.5d0
       if(OPT_norm)then
       s=s+vertex(l,i)**2
       end if
       end do!l

       if(OPT_norm)then
       rs=rpr/dsqrt(s)
       do l=1,3
       vertex(l,i)=vertex(l,i)*rs
       end do!l
       end if
       end do!i

       nvert3=nvertmx 
       do i=nvert2+1,nvert3
       s=0.0d0
       do l=1,3
       vertex(l,i)=(vertex(l,vertex_parent(1,i))+
     &              vertex(l,vertex_parent(2,i)))*0.5d0
       if(OPT_norm)then
       s=s+vertex(l,i)**2
       end if
       end do!l
        if(OPT_norm)then
       rs=rpr/dsqrt(s)
       do l=1,3
       vertex(l,i)=vertex(l,i)*rs
       end do!l
       end if
       end do!i

       do i=1,ntrimx
       s=0.0d0
       do k=1,3
       vdot(k,i)=vertex(k,tri_parent(1,i))+
     &           vertex(k,tri_parent(2,i))+
     &           vertex(k,tri_parent(3,i))
       s=s+vdot(k,i)**2
       end do!k

       s=1.0d0/dsqrt(s)
       rs=rpr*s

       do k=1,3
       vndot(k,i)=vdot(k,i)*s
       vdot(k,i)=vdot(k,i)*rs
       end do!k

       if(.not.OPT_AreaEachDot)then
       area_dot(i)=ar
       else

      area_dot(i)=curv_triang_area(vertex(1,tri_parent(1,i)),
     &	                           vertex(1,tri_parent(2,i)),
     &                             vertex(1,tri_parent(3,i)),
     &                             center,rpr)

       end if
       end do!i

c 
       return
       end
c
       subroutine gener_cc4(v1,v2,v3,rpr,area,ndot,vdot,area_dot,vndot)

       implicit none
       integer ndotccmx
       parameter(ndotccmx=128)
       integer ndot
       real*8 v1(3),v2(3),v3(3),rpr,area
       real*8 vdot(3,ndotccmx),area_dot(ndotccmx),vndot(3,ndotccmx)
c real function:
       real*8 curv_triang_area

c 
       integer ntrimx        !number of dots-triangles
       parameter (ntrimx=64)
       integer nvertmx
       parameter (nvertmx=3+42) !number of vertexes

       real*8 one3,s,rs,ar,vertex(3,nvertmx)
       real*8 center(3)
       integer vertex_parent(2,nvertmx)
       integer nvert1,nvert2,nvert3,nvert4
       integer tri_parent(3,ntrimx)
       integer i,j,k,l
       logical OPT_AreaEachDot

       data center/0.0d0,0.0d0,0.0d0/
       data vertex_parent/1,1,2,2,3,3,        ! lev1  vert 1,2,3
     & 1,2,2,3,1,3,                         ! lev2 - vert 4,5,6
     & 1,4,1,6,4,6,4,5,5,6,3,5,3,6,2,4,2,5, ! lev3-vert 7,8,9,10,11,12,13,14,15
     & 9,10, 10,11, 9,11, 6,9, 6,11,        ! lev4      16,17,18,19,20
     & 4,9, 4,10, 5,11, 5,10, 6,8,          ! lev4      21,22,23,24,25
     & 8,9, 7,9, 4,7, 7,8, 1,8,             ! lev4      26,27,28,29,30
     & 1,7, 6,13, 11,13, 12,13, 11,12,      ! lev4      31,32,33,34,35
     & 5,12, 3,12, 3,13, 4,14, 10,14,       ! lev4      36,37,38,39,40
     & 10,15, 14,15, 5,15,2,14, 2,15 /      ! lev4      41,42,43,44,45

       data tri_parent/16,17,18, 9,16,18, 11,17,18, 10,16,17, !lev4; 64 triangl
     &                 18,19,20, 9,18,19, 11,18,20, 6,19,20,
     &                 21,22,16, 9,16,21, 10,16,22, 4,21,22,
     &                 17,23,24, 10,17,24, 11,17,23, 5,23,24,

     &                 19,25,26, 6,19,25, 9,19,26, 8,25,26,
     &                 9,21,27, 21,27,28, 4,21,28, 7,27,28,
     &                 9,26,27, 8,26,29, 26,27,29, 7,27,29,
     &                 8,29,30, 7,29,31, 29,30,31, 1,30,31,
    
     &                 6,20,32, 20,32,33, 13,32,33, 11,20,33,
     &                 13,33,34, 11,33,35, 33,34,35, 12,34,35,
     &                 11,23,35, 23,35,36, 12,35,36, 5,23,36,
     &                 13,34,38, 34,37,38, 12,34,37, 3,37,38,

     &                 22,39,40, 10,22,40, 4,22,39, 14,39,40,
     &                 10,40,41, 14,40,42, 15,41,42, 40,41,42,
     &                 10,24,41, 5,24,43,  24,41,43, 15,41,43,
     &                 42,44,45, 14,42,44, 15,42,45, 2,44,45/  !lev4 
       
       
       
       OPT_AreaEachDot=.true. 
      
       nvert1=3
       nvert2=6
       nvert3=15
       nvert4=nvertmx

       ndot=ntrimx
       ar=area/dfloat(ndot)
       one3=1.0d0/3.0d0

c generate all vertex
       do l=1,nvert1
       vertex(l,1)=v1(l)
       vertex(l,2)=v2(l)
       vertex(l,3)=v3(l)
       end do !l

       do i=nvert1+1,nvert2
       s=0.d0
       do l=1,3
       vertex(l,i)=(vertex(l,vertex_parent(1,i))+
     &              vertex(l,vertex_parent(2,i)))*0.5d0
       s=s+vertex(l,i)**2
       end do!l
       rs=rpr/dsqrt(s)
       do l=1,3
       vertex(l,i)=vertex(l,i)*rs
       end do!l
       end do!i

       
       do i=nvert2+1,nvert3
       s=0.d0
       do l=1,3
       vertex(l,i)=(vertex(l,vertex_parent(1,i))+
     &              vertex(l,vertex_parent(2,i)))*0.5d0
       s=s+vertex(l,i)**2
       end do!l
       rs=rpr/dsqrt(s)
       do l=1,3
       vertex(l,i)=vertex(l,i)*rs
       end do!l
       end do!i


       
       do i=nvert3+1,nvert4
       s=0.d0
       do l=1,3
       vertex(l,i)=(vertex(l,vertex_parent(1,i))+
     &              vertex(l,vertex_parent(2,i)))*0.5d0
       s=s+vertex(l,i)**2
       end do!l
       rs=rpr/dsqrt(s)
       do l=1,3
       vertex(l,i)=vertex(l,i)*rs
       end do!l
       end do!i

       do i=1,ntrimx
       s=0.0d0
       do k=1,3
       vdot(k,i)=vertex(k,tri_parent(1,i))+
     &           vertex(k,tri_parent(2,i))+
     &           vertex(k,tri_parent(3,i))
       s=s+vdot(k,i)**2
       end do!k

       s=1.0d0/dsqrt(s)
       rs=rpr*s

       do k=1,3
       vndot(k,i)=vdot(k,i)*s
       vdot(k,i)=vdot(k,i)*rs
       end do!k

       if(.not.OPT_AreaEachDot)then
       area_dot(i)=ar
       else

      area_dot(i)=curv_triang_area(vertex(1,tri_parent(1,i)),
     &	                           vertex(1,tri_parent(2,i)),
     &                             vertex(1,tri_parent(3,i)),
     &                             center,rpr)

       end if

       end do!i

c 
       return
       end
c

	 subroutine gener_conc_dot02(UP,ARUP,TETP,PrPIJK,
     &	 v1,v2,v3,rpr,dens,
     &	 area_cc,vdot,area_dot,vndot,NUP,ndot)

         implicit none
	 real*8 UP(3,*),ARUP(*),TETP(*)
	 real*8 PrPIJK(3)
	 integer NUP
         integer ndotccmx
         parameter(ndotccmx=128)  !the same size as in MAIN
         integer ndot
         real*8 v1(3),v2(3),v3(3),rpr,dens
         real*8 vdot(3,*),area_dot(*),vndot(3,*)
	 real*8 area_cc

c local variables
	 real*8 G(3,3),x(3),y(3),z(3),u(3)
         real*8 v11(3),v21(3),v31(3),z1(3)
         real*8 center(3)
	 real*8 aa,bb,cc
	 real*8 s,s1,s2,s3,tetmx
	 real*8 null,sign
	 real*8 area
         real*8 toler_z
c real function:
         real*8 curv_triang_area
	 real*8 DDOT

c 
	 logical CONTROL
         integer i,k,j,l
	 data center/0.0d0,0.0d0,0.0d0/

	 CONTROL = .false.
	 null = 0.0d0
         toler_z = (0.050d0/dsqrt(dens))/rpr  ! minimal distance of dot from
c                                            ! triangle adges

	 if(CONTROL)then
	 write(*,*)'gener_CC02'
	 write(*,'(a6,3f10.6)')'v1=',v1
	 write(*,'(a6,3f10.6)')'v2=',v2
	 write(*,'(a6,3f10.6)')'v3=',v3
	 end if


        s=0.0d0
        do k=1,3
          aa=v1(k)-center(k)
          bb=v2(k)-center(k)
          cc=v3(k)-center(k)
	  z(k)=aa+bb+cc
	  s=s + z(k)**2
        enddo
	s = dsqrt(s)
	do k=1,3
	z(k) = z(k)/s
	G(k,3) = z(k)
        z1(k)=z(k)*rpr
	end do
        
	s1=0.0d0
	do k=1,3
        s1 = s1 + z(k)*v1(k)
	end do
	s2=0.0d0
	do k=1,3
        s2 = s2 + z(k)*v2(k)
	end do
	s3=0.0d0
	do k=1,3
        s3 = s3 + z(k)*v3(k)
	end do
	tetmx = s1
	if(tetmx.gt.s2)tetmx=s2
	if(tetmx.gt.s3)tetmx=s3
        tetmx = tetmx/rpr
	tetmx = dacos(tetmx)       

        if(CONTROL)then
	write(*,*)'tetmx = ', tetmx
	end if

        if(s1.le.s2.and.s1.le.s3)then
	s=null
	do k=1,3
	x(k) = v1(k) - z(k)*s1
	s = s + x(k)**2
	end do
	s = dsqrt(s)
	do k=1,3
	x(k) = x(k)/s
	G(k,1) = x(k)
	end do 
        end if
        if(s2.le.s1.and.s2.le.s3)then
	s=null
	do k=1,3
	x(k) = v2(k) - z(k)*s2
	s = s + x(k)**2
	end do
	s = dsqrt(s)
	do k=1,3
	x(k) = x(k)/s
	G(k,1) = x(k)
	end do 
        end if
        if(s3.le.s1.and.s3.le.s2)then
	s=null
	do k=1,3
	x(k) = v3(k) - z(k)*s3
	s = s + x(k)**2
	end do
	s = dsqrt(s)
	do k=1,3
	x(k) = x(k)/s
	G(k,1) = x(k)
	end do 
        end if

c y= z*x
	call DVECTPR(z,x,y)
	do k=1,3
	G(k,2) = y(k)
	end do
	if(CONTROL)then
	write(*,*)'Rotatiom Matr G:'
	 write(*,'(a6,3f10.6)')'G(k,1)=',(G(k,1), k=1,3)
	 write(*,'(a6,3f10.6)')'G(k,2)=',(G(k,2), k=1,3)
	 write(*,'(a6,3f10.6)')'G(k,3)=',(G(k,3), k=1,3)
	 end if

        do k=1,3
        v11(k) = v1(k) + (z1(k)-v1(k))*toler_z
        v21(k) = v2(k) + (z1(k)-v2(k))*toler_z
        v31(k) = v3(k) + (z1(k)-v3(k))*toler_z
        end do!k

	call DVECTPR(v21,v31,x)
	call DVECTPR(v31,v11,y)
	call DVECTPR(v11,v21,z)
	 
        sign = DDOT(v11,x)

	ndot = 0
	area = null

	do i=1,NUP

	do k=1,3
	u(k)=null
	do l=1,3
	u(k)=u(k) + G(k,l)*UP(l,i)
	end do
	end do !k
c
        s1=DDOT(u,x) 
	s2=DDOT(u,y) 
	s3=DDOT(u,z) 
	
        if(s1*sign.gt.null.and.s2*sign.gt.null.and.
     &     s3*sign.gt.null)then       
c
	 ndot = ndot +1
	 do k=1,3
	 vndot(k,ndot) = -u(k)/rpr
	 vdot(k,ndot) = u(k)            
	 end do !k

	 area_dot(ndot)= ARUP(i)
	 area=area + area_dot(ndot)
	 if(CONTROL)then
	 write(*,*)'dot accepted i=',i,'tet(i)=',TETP(i)
	 end if

	 end if ! INside

	 if(TETP(i).gt.tetmx)goto 1001 ! max teta 
	 end do !i

1001     continue         

	 if(ndot.lt.1)then
	 write(*,*)'gener_conc02:ERROR: NO dots on CC face !!'
         STOP  4022
	 end if

	 area_cc=curv_triang_area(v1,v2,v3,center,rpr)
	 if(CONTROL)then
	 write(*,*)'Analcurv_triang_area=',area_cc
	 write(*,*)'Num      triang_area=',area
	 write(*,*)'ndot =',ndot
	 end if

	 if(area.ne.null)then
         aa = area_cc/area
	 do i=1,ndot
         area_dot(i) = area_dot(i)*aa
	 end do
	 end if

	if(CONTROL)then
	write(*,*)'dot i, xyz            vnorm   area'
	do i=1,ndot
	write(*,'(i5,1x,3f10.6,2x,3f10.6,2x,f10.7)')i,
     &	(vdot(k,i),k=1,3),(vndot(k,i),k=1,3),area_dot(i)
	end do
	end if

	 return
	 end
c
       subroutine arc_points(ri,rj,dij,hij,rpr,dens,putsm,narc,ni,nj,
     & densI,densJ,v0arc,stor,arc_cross,dot_cross)


       implicit none
       integer narcmx
       parameter(narcmx=100) ! =MAXARC dens <16 for Rat<3

       integer putsm    !if 2/1/ then at least 2/1 dot will be put on
			!for any arc lenght
       integer narc,ni,nj
       real*8 v0arc(3,narcmx)
       real*8 ri,rj,dij,hij,rpr,dens
       real*8 densI,densJ,stor
       logical dot_cross(narcmx)
       logical arc_cross
c
	 real*8 PI2,null,onehalf,one,two,three,four,big,small
	 common/CONSTANTI/PI2,null,onehalf,one,two,three,four,big,small

	 real*8  toler_nb,toler_pr,toler_yon,toler_d,toler_cx,toler_ovr 
	 real*8  ktoler_yon
	 common/TOLERANCE/toler_nb,toler_pr,toler_yon,toler_d,
     &	 toler_cx,toler_ovr,ktoler_yon

c inner variables
       real*8 teti,tetj,dens2,dtet2
       real*8 di,dj,di2,dj2,tet,tetmx,dtet
       real*8 pi,cost,cosdt,rip,rjp,aa
       real*8 step,bb
       real*8 toler_cross
       logical changeSg
       logical testpr
       integer i,j,k,n
       integer n1,n2,n3

	testpr=.false.   !test print

        pi=PI2       !3.1415927d0

        dens2=rpr*dsqrt(dens)
	rip=ri+rpr
	rjp=rj+rpr
	stor=null
	toler_cross=0.75d0   
  

	if(ri.gt.rj)then
	cost=hij/rip
	changeSg=.false.
	else
	cost=hij/rjp
	changeSg=.true.
	end if

	tetmx=dacos(cost)

	cosdt=(rip**2 + rjp**2 - dij**2)/(2.0d0*rip*rjp)
	dtet=dacos(cosdt)

	if(testpr)then
	write(*,*)'ARC: tetmx, dtet=',tetmx,dtet
        end if

 
	dtet2=dtet*0.5d0
        
	ni=0
	nj=0
	if(putsm.eq.2)then
	ni=DINT(dtet2*dens2+0.5d0)
	if(ni.eq.0)ni=1
	narc=2*ni
	nj=ni
	
	else

	narc=DINT(dtet*dens2+0.5d0)
        if(narc.eq.0.and.putsm.eq.1)narc=1
	nj=narc/2
	ni=narc-nj
	if(changeSg)then
	ni=narc/2
	nj=narc-ni
	end if
	end if

	if(narc.lt.1)goto 1001

         if(narc.gt.narcmx)then
         write(*,*)'ERROR : sub: nrc_points , narcmx is small=',narcmx
          STOP  4018
         end if
        
	di=dtet/dfloat(narc)
	di2=di*0.5d0

        densI=di
	densJ=densI

	toler_cross=ktoler_yon*di*rpr*toler_cross !dot size dependent
	toler_cross=ktoler_yon    !Aug.6

	tet=tetmx+di2

	if(changeSg)then
	tet=tetmx - dtet
	tet=tet-di2
	di=-di
	end if !changeSg


	do i=1,narc
	tet=tet - di
	v0arc(1,i)= -rpr*dsin(tet)
	if(changeSg)v0arc(1,i)=-v0arc(1,i)
	v0arc(2,i)= hij - rpr*dcos(tet)
        v0arc(3,i)= 0.0d0
	if(v0arc(2,i).le.toler_cross)then
	dot_cross(i)=.true.
	else
	dot_cross(i)=.false.
	end if
	end do !i

	 arc_cross=.false.
         if(hij.le.rpr + toler_cross)arc_cross=.true.

	 if(arc_cross)then
	 step=0.05d0
         n=int(dtet/step) +1 
	di=dtet/dfloat(n) 
	di2=di*0.5d0
        tet=tetmx+di2
	aa=0.0d0
	do i=1,n
	tet=tet-di
	bb=hij-rpr*dcos(tet)

	if(bb.le.toler_cross)bb=null  

	aa=aa+bb
	end do
	aa=aa*di
	stor=2.0d0*pi*rpr*aa
	end if !arc_cross
       
	if(.not.arc_cross)then
	stor=hij*dtet-rpr*(dsin(tetmx)-dsin(tetmx-dtet))
        stor=2.0d0*pi*rpr*stor
	end if !

	n1=0
	n2=0
	n3=0
	do i=1,narc
	if(.not.dot_cross(i))then
	n3=n3+1
	if(i.le.ni)then
	n1=n1+1
	else
	n2=n2+1
	end if
	dot_cross(n3)=dot_cross(i)
	do k=1,3
	v0arc(k,n3)=v0arc(k,i)
	end do!k
	end if !not.dot_cross
	end do !i
	ni=n1
	nj=n2
	narc=n3

1001	return
	end

c

      	subroutine torus_rot_angl(iatm_mx,nbr_mx,nconc_mx,
     &                  iat,jat,UIJ,HIJ,nvertex,atvert_conc,cvertex,
     &                  cprobe_ijk,conc_gl_ijk,                 
     &                  icfIJ,concfS_ij,RotAng,SignRotS,AIJarcS,GarcS)

       implicit none

       integer iatm_mx,nbr_mx,nconc_mx
       integer iat,jat
       integer nvertex(iatm_mx)
       integer atvert_conc(nbr_mx,iatm_mx)
       real*8 UIJ(3),HIJ
       real*8 cvertex(3,nbr_mx,iatm_mx)
       real*8 cprobe_ijk(3,nconc_mx)
       integer conc_gl_ijk(6,nconc_mx)
       integer concfS_ij(nbr_mx) ! sorted by increase of the tor_angl_rot 

c inner variables
       integer nbr_mxx
       parameter(nbr_mxx=128)

       integer order(nbr_mxx)
       integer i,j,iperm
       integer k,katt,kpos,ki,kj
       integer iavert,javert,kavert
       integer IATOM,JATOM,KATOM
       integer OPT_printsd,iiprint0,iiprint1,iiprint2
       integer iconcf,icf
       integer concf_ij(nbr_mxx) ! sorted by increase of the tor_angl_rot 

       real*8 VPI(3),VPJ(3),VPK(3),TVP(3),PIJK(3)
       real*8 UQ1(3),UT1(3),vvv(3)
       real*8 AIJarc(3,nbr_mxx)
       real*8  Garc(3,3,nbr_mxx)
       integer SignRot(nbr_mxx)

       real*8 SIGN,aa1,aa2
       real*8 PI
       real*8 toler

       logical exist_i,exist_j,exist_k

c
	 real*8 PI2,null,onehalf,one,two,three,four,big,small
	 common/CONSTANTI/PI2,null,onehalf,one,two,three,four,big,small

c RESULT
       integer icfIJ
       real*8 AIJarcS(3,nbr_mx)
       real*8 GarcS(3,3,nbr_mx)
       real*8 RotAng(nbr_mx)
       integer SignRotS(nbr_mx)

       logical OPT_printwarn
c
c function
       real*8 DDLENG
       real*8 DDOT

	PI=PI2             !dacos(-1.0d0) ! 3.1415927d0
	toler=1.0d-6
	iiprint0=0
	iiprint1=1
	iiprint2=2

c	OPT_printsd=1 ! test print
        OPT_printsd=0 ! no print

	OPT_printwarn = .true.
	 

c initialize
       IATOM=iat
       JATOM=jat
       icf=0
       do i=1,nbr_mxx
       concf_ij(i)=0
       order(i)=0
       end do


       if(OPT_printsd.ge.iiprint1)then
       write(*,*)'*****************************************'
       write(*,*)'START SUB torus_angl_rot'
       write(*,*)'nvertex(IATOM)=',nvertex(IATOM),
     &    ' nvertex(JATOM)=',nvertex(JATOM)
       end if
c
c
       if(nvertex(IATOM).ge.1.and.nvertex(JATOM).ge.1)then !Do i LOOP
       do i=1,nvertex(IATOM)

       iconcf=atvert_conc(i,IATOM)  !conc face glob nmb

       do j=1,nvertex(JATOM)
       if(iconcf.eq.atvert_conc(j,JATOM))then !MAIN calculation

       icf=icf+1

c control
       if(icf.gt.nbr_mx.or.icf.gt.nbr_mxx)then
       write(*,*)'Sub torus_rot_angl:ERROR: nbr_mx,nbr_mxx SMALL'
        STOP  4019
       end if! control

       concf_ij(icf)=iconcf
       iavert=i
       javert=j

              
	       if(OPT_printsd.ge.iiprint1)then
	       write(*,*)
	       write(*,*)'   icf =  ', icf
	       write(*,*)'Vertex Iat,Jat=',iavert,javert
               end if


	do k=1,3
	if(conc_gl_ijk(k,iconcf).eq.IATOM)then
	ki=k
	exist_i=.true.
	end if 
	end do

	do k=1,3
	if(conc_gl_ijk(k,iconcf).eq.JATOM.and.k.ne.ki)then
	kj=k
	exist_j=.true.
	end if 
	end do

	do k=1,3
	if(k.ne.ki.and.k.ne.kj.and.exist_i.and.exist_j)then
	exist_k=.true.
	kpos=k
	KATOM=conc_gl_ijk(k,iconcf)
	end if
	end do!k

c check
	if(.not.exist_i.and.exist_j.and.exist_k)then
	write(*,*)'atom K is not found'
        STOP  4020
	end if


	kavert=conc_gl_ijk(3+kpos,iconcf)

        do k=1,3
	VPI(k)=cvertex(k,iavert,IATOM)
	VPJ(k)=cvertex(k,javert,JATOM)
	VPK(k)=cvertex(k,kavert,KATOM)
	PIJK(k)=cprobe_ijk(k,iconcf)
        UQ1(k)= -VPI(k)-VPJ(k)
	TVP(k)= UQ1(k)- VPK(k)
        end do!k

c 
	      if(OPT_printsd.ge.iiprint1)then
	write(*,*)'ARC  from iconcf=', iconcf
	write(*,*)'IATOM,JATOM,KATOM=',IATOM,JATOM,KATOM
	write(*,*)'iavert,javert,kavert=',iavert,javert,kavert
	write(*,*)'VERTEX VPI=',VPI
	write(*,*)'VERTEX VPJ=',VPJ
	write(*,*)'VERTEX VPK=',VPK
	write(*,*)'PROBE PIJK=',PIJK
              end if
c 
	       aa1=DDOT(UIJ,UQ1)
	       do k=1,3
	       UQ1(k)=UQ1(k)-aa1*UIJ(k)
	       end do!k

	       aa1= DDLENG(UQ1)
	       do k=1,3
	       UQ1(k)=UQ1(k)/aa1
	       AIJarc(k,icf)=HIJ*UQ1(k) 
	       end do!k

	       call DVECTPR(UIJ,UQ1,UT1)

              do K = 1,3
                 Garc(K,1,icf) = UIJ(K)
                 Garc(K,2,icf) = UQ1(K)
                 Garc(K,3,icf) = UT1(K)
              end do !k
c 
	      if(OPT_printsd.ge.iiprint1)then
	write(*,*)'FRAME : '
	write(*,*)'UIJ=',UIJ
	write(*,*)'UQ1=',VPJ
	write(*,*)'UT1=',UT1
	      end if !OPT_print

	      SIGN = DDOT(TVP,UT1)

	      if(SIGN.lt.null)then
	      SignRot(icf)=-1
	      else
	      SignRot(icf)=1
	      end if

	      if(OPT_printsd.ge.iiprint1)then
	      write(*,*)'SIGN of ARC ROTATION=',SignRot(icf)
	      write(*,*)'TVP=',TVP
	      write(*,*)'UT1_icf=',UT1
	      write(*,*)'UT1_1  =',(Garc(K,3,1),K=1,3)
	      end if


	      RotAng(1)=null

	      if(icf.gt.1)then 

	      aa1=DDOT(Garc(1,3,1),UT1)  ! cos(RotAng(1-->icf))

	      if(aa1.gt.one)then
	      if(OPT_printwarn)then
	      write(*,*)'WARNING in torus_ang_rot:aa1>one',aa1
	      end if
	      aa1 = one
	      end if
	      if(aa1.lt.-one)then
	      if(OPT_printwarn)then
	      write(*,*)'WARNING in torus_ang_rot:aa1<-one',aa1
	      end if
	      aa1 = -one
	      end if

	      RotAng(icf)=dacos(aa1)

              call DVECTPR(Garc(1,2,1),UQ1,vvv)

	      aa2=DDOT(UIJ,vvv)
c	      write(*,*)'DDOT(UIJ,vvv)=',aa2
 
	      aa2=aa2*SignRot(1)

	      if(aa2.lt.-toler)
     &	      RotAng(icf)=two*PI-RotAng(icf)

               if(OPT_printsd.ge.iiprint1)then
	       write(*,*)'icf=', icf
	       write(*,*)'COS(rotangl)=',aa1,' RotAng=',RotAng(icf)
	       write(*,*)'SignRot(1)*aa2=',aa2
	       write(*,*)'CORRECTED RotAng(icf)=',RotAng(icf)
	       end if

	      end if

	      end if !MAIN
	      end do!j
	      end do!i
	      end if !i LOOP

	      icfIJ=icf

c control
	    
	      if(icfIJ.ne.2*(icfIJ/2))then
	      write(*,*)'SUbr torus_rot_ang: ERROR BAD icfIJ=',icfIJ
              STOP  4021
	      end if

c 
	     do i=1,icfIJ
	     order(i)=i
	     end do!i

            if(OPT_printsd.ge.iiprint1)then
            write(*,*)'Initial RotAng '
	    do i=1,icfIJ
	    write(*,*)i,RotAng(i),order(i)
	    end do
            end if

5000        iperm=0

	     do i=2,icfIJ
	     if(RotAng(i).lt.RotAng(i-1))then
	     iperm=iperm+1
	     aa1=RotAng(i-1)
	     RotAng(i-1)=RotAng(i)
	     RotAng(i) = aa1
	     j=order(i-1)
	     order(i-1)=order(i)
	     order(i) = j
	     end if
	     end do !i

	     if(iperm.ge.1)goto 5000


            if(OPT_printsd.ge.iiprint1)then    
            write(*,*)'SORTED RotAng '
	    do i=1,icfIJ
	    write(*,*)i,RotAng(i),order(i)
	    end do
	    end if

	    do i=1,icfIJ
	    j=order(i)
            SignRotS(i)=SignRot(j) 
              do K = 1,3
	         AIJarcS(K,i) = AIJarc(K,j)
                 GarcS(K,1,i) = Garc(K,1,j)
                 GarcS(K,2,i) = Garc(K,2,j)
                 GarcS(K,3,i) = Garc(K,3,j)
              end do !k
	      concfS_ij(i)=concf_ij(j)  ! PIJK's probe positions glob numbers
	      end do !i

c 
        if(OPT_printsd.ge.iiprint1)then 
       write(*,*)'END SUB torus_angl_rot'
       write(*,*)'*****************************************'
       end if

            return
	    end

c 
           subroutine  SwriteA(i1,i2,i3,ar1,ar2,lg1,lg2,nn,Ai,Ar,Al)

	   implicit none
	   integer nnmx
	   parameter (nnmx=100) ! nnmx=MAXNBR
	   real*8 ar1(3),ar2(3)
	   real*8 Ar(6,nnmx)
	   integer*2 i1,i2
           integer*2 i3
	   integer nn,k
	   integer*2 Ai(3,nnmx)
	   logical*1 lg1,lg2
	   logical*1 Al(2,nnmx)

	   Ai(1,nn)=i1
	   Ai(2,nn)=i2
	   Ai(3,nn)=i3

	   do k=1,3
	   Ar(k,nn)=ar1(k)
	   Ar(k+3,nn)=ar2(k)
	   end do

	   Al(1,nn)=lg1
	   Al(2,nn)=lg2

	   return
	   end
c
           subroutine  SwriteB(i1,i2,i3,ar1,a2,lg1,np,nn,Ai,Ar,Al)

	   implicit none
	   integer npm
	   parameter (npm=128)  ! npm=ndotccmx=128 !
	   integer nnmx
	   parameter (nnmx=100) ! nnmx=MAXNBR

	   real*8 ar1(3),a2
	   real*8 Ar(4,npm,nnmx)
	   integer*2 i1,i2,i3
	   integer nn,np,k
	   integer*2 Ai(3,npm,nnmx)
	   logical*1 lg1
	   logical*1 Al(npm,nnmx)

	   Ai(1,np,nn)=i1
	   Ai(2,np,nn)=i2
	   Ai(3,np,nn)=i3

	   do k=1,3
	   Ar(k,np,nn)=ar1(k)
	   end do
	   Ar(4,np,nn)=a2

	   Al(np,nn)=lg1

	   return
	   end
c
           subroutine  SwriteA4(nn,Ai,Ar,Al)

	   implicit none
	   integer nnmx
	   parameter (nnmx=100) ! nnmx=MAXNBR
	   real*8 Ar(6,nnmx)
	   integer nn,k
	   integer*2 Ai(3,nnmx)
	   logical*1 Al(2,nnmx)

	   write(4) (Ai(k,nn),k=1,3),(Ar(k,nn),k=1,6),Al(1,nn),Al(2,nn)

	   return
	   end

           subroutine  SwriteB4(np,nn,Ai,Ar,Al)

	   implicit none
	   integer npm
	   parameter (npm=128)  ! npm=ndotccmx=128 !
	   integer nnmx
	   parameter (nnmx=100) ! nnmx=MAXNBR

	   real*8 Ar(4,npm,nnmx)
	   integer nn,np,k,ip
	   integer*2 Ai(3,npm,nnmx)
	   logical*1 Al(npm,nnmx)

	    if(np.ge.1)then
	    do ip=1,np
	   write(4) (Ai(k,ip,nn),k=1,3),(Ar(k,ip,nn),k=1,4),Al(ip,nn)
            end do
	    end if

	   return
	   end

           subroutine  SwriteA_WP(nnmx,nn,Ai,Ar,Al,nwpmx,nwp,
     &	   WPi,WPr,WPl)

	   implicit none
	   integer nnmx,nwpmx
	   real*8 Ar(6,nnmx)
	   integer nn,nwp
	   integer*2 Ai(3,nnmx)
	   logical*1 Al(2,nnmx)
	   real*8 WPr(6,nwpmx)
	   integer*2 WPi(3,nwpmx)
	   logical*1 WPl(2,nwpmx)
	   integer k
     
           do k=1,3
           WPi(k,nwp)=Ai(k,nn)
	   end do
	   do k=1,6
	   WPr(k,nwp)=Ar(k,nn)
	   end do
	   WPl(1,nwp)=Al(1,nn)
	   WPl(2,nwp)=Al(2,nn)

	   return
	   end

           subroutine  SwriteA_WD(nnmx,np,nn,Ai,Ar,Al,
     &	   ndotmx,nst,WDi,WDr,WDl)

	   implicit none
	   integer npm
	   parameter (npm=128)  ! npm=ndotccmx=128 !

	   integer nnmx,ndotmx

	   real*8 Ar(4,npm,nnmx)
	   integer nn,np,nst
	   integer*2 Ai(3,npm,nnmx)
	   logical*1 Al(npm,nnmx)

	   real*8 WDr(4,ndotmx)
	   integer*2 WDi(3,ndotmx)
	   logical*1 WDl(ndotmx)
           integer k,ip,nnew

	    if(np.ge.1)then

	   do ip=1,np
	   nnew=nst+ip
           do k=1,3
           WDi(k,nnew)=Ai(k,ip,nn)
           end do
	   do k=1,4
	   WDr(k,nnew)=Ar(k,ip,nn)
	   end do
	   WDl(nnew)=Al(ip,nn)
           end do
	   end if

	   return
	   end

           subroutine  SwriteWP(i1,i2,i3,ar1,ar2,lg1,lg2,
     &	   probmx,nn,WPi,WPr,WPl)

	   implicit none
	   integer probmx
	   real*8 ar1(3),ar2(3)
	   real*8 WPr(6,probmx)
	   integer*2 i1,i2
           integer*2 i3
	   integer*2 WPi(3,probmx)
	   logical*1 lg1,lg2
	   logical*1 WPl(2,probmx)
	   integer nn,k

	   WPi(1,nn)=i1
	   WPi(2,nn)=i2
	   WPi(3,nn)=i3

	   do k=1,3
	   WPr(k,nn)=ar1(k)
	   WPr(k+3,nn)=ar2(k)
	   end do

	   WPl(1,nn)=lg1
	   WPl(2,nn)=lg2

	   return
	   end
c
           subroutine  SwriteWD(i1,i2,i3,ar1,a2,lg1,
     &	   ndotmx,nn,WDi,WDr,WDl)
	   implicit none
            
           integer ndotmx
	   real*8 ar1(3),a2
	   real*8 WDr(4,ndotmx)
	   integer*2 i1,i2,i3
	   integer*2 WDi(3,ndotmx)
	   logical*1 lg1
	   logical*1 WDl(ndotmx)
           integer nn
           integer k

	   WDi(1,nn)=i1
	   WDi(2,nn)=i2
	   WDi(3,nn)=i3

	   do k=1,3
	   WDr(k,nn)=ar1(k)
	   end do
	   WDr(4,nn)=a2

	   WDl(nn)=lg1

ctt  write
ctt           write(*,'(a10,4i5,1x,4f8.4)')'in SwriteWD ndot',nn,
ctt     &                                   i1,i2,i3,ar1,a2

	   return
	   end
c
           subroutine  SreadWP(i1,i2,i3,ar1,ar2,lg1,lg2,
     &	   probmx,nn,WPi,WPr,WPl)

	   implicit none
	   integer probmx
	   real*8 ar1(3),ar2(3)
	   real*8 WPr(6,probmx)
	   integer*2 i1,i2,i3
	   integer*2 WPi(3,probmx)
	   logical*1 lg1,lg2
	   logical*1 WPl(2,probmx)
	   integer nn,k

	   i1=WPi(1,nn)
	   i2=WPi(2,nn)
	   i3=WPi(3,nn)

	   do k=1,3
	   ar1(k)=WPr(k,nn)
	   ar2(k)=WPr(k+3,nn)
	   end do

	   lg1=WPl(1,nn)
	   lg2=WPl(2,nn)

	   return
	   end
c
           subroutine  SreadWD(i1,i2,i3,ar1,a2,lg1,
     &	   ndotmx,nn,WDi,WDr,WDl)
	   implicit none

	   integer ndotmx
	   real*8 ar1(3),a2
	   real*8 WDr(4,ndotmx)
	   integer*2 i1,i2,i3
	   integer*2 WDi(3,ndotmx)
	   logical*1 lg1
	   logical*1 WDl(ndotmx)

           integer nn,k

	   i1=WDi(1,nn)
	   i2=WDi(2,nn)
	   i3=WDi(3,nn)

	   do k=1,3
           ar1(k)=WDr(k,nn)
	   end do
	   a2=WDr(4,nn)

	   lg1=WDl(nn)

ctt  write
ctt           write(*,'(a10,4i5,1x,4f8.4)')'in SreadWD ndot',nn,
ctt     &                                   i1,i2,i3,ar1,a2

	   return
	   end
c
c

	subroutine remove_ccdot(nprob_mx,ndot_mx,nconcij_mx,nconc_mx,
     &                  icfIJ,
     &                  RP,BIJ,UIJ,HIJ,
     &                  cprobe_ijk,concf_ijf,nconc_glb_nprpos,
     &                  start_dotn,WDScAr,WDYON,WDYONRm)

       implicit none
c subroutine parameters
       integer nprob_mx,ndot_mx,nconc_mx,nconcij_mx
       real*8 BIJ(3)   

       real*8 UIJ(3)   
       real*8 RP,HIJ
       real*8 cprobe_ijk(3,nconc_mx)
       integer concf_ijf(nconcij_mx)
       integer nconc_glb_nprpos(nconc_mx)
       integer icfIJ                 
   
       integer start_dotn(nprob_mx)      
  
       real*8 WDScAr(4,ndot_mx)
       logical*1 WDYON(ndot_mx)

       logical*1 WDYONRm(ndot_mx)   
c
	 real*8 PI2,null,onehalf,one,two,three,four,big,small
	 common/CONSTANTI/PI2,null,onehalf,one,two,three,four,big,small

	 real*8  toler_nb,toler_pr,toler_yon,toler_d,toler_cx,toler_ovr 
	 real*8  ktoler_yon
	 common/TOLERANCE/toler_nb,toler_pr,toler_yon,toler_d,
     &	 toler_cx,toler_ovr,ktoler_yon

c
       real*8 Hpmx        ! distance from the torus axis to probe arc_ij
       real*8 xpm         ! distance from the torus center to the plane 
c                           perpendicular to Uij torus axis


	integer ic,jc,id,k
	integer icgl,jcgl
	integer nst,nfin,nprb
	real*8 aa,dist
	real*8 xyzd(3),xyzdp(3)
	real*8 toler_ccr
	real*8 toler_ccr2
	real*8 ktoler

        ktoler=ktoler_yon*0.5d0*0.5d0 ! 0.5 - average for sin(tet)
        ktoler=ktoler_yon*0.5d0 !Ag.5  0.5 - average for sin(tet)
     
       if(icfIJ.ge.1)then 
       do ic=1,icfIJ

       icgl=concf_ijf(ic)    ! glb number of CC face
       nprb=nconc_glb_nprpos(icgl)    ! glb probe nmb
       nst = start_dotn(nprb)      ! start
       nfin= start_dotn(nprb+1)    ! final number for CC dots on this face

       if((nst.ge.1).and.(nfin.ge.1))then
       do id=nst,nfin
       if(WDYON(id).and.(.not.WDYONRm(id)))then

       xpm = null
       do k=1,3
       xyzd(k)=WDScAr(k,id)
       xpm = xpm +UIJ(k)*(xyzd(k)-BIJ(k))
       end do !k

       Hpmx = RP*RP - xpm*xpm

       toler_ccr = ktoler*dsqrt(WDScAr(4,id))/RP !Ag.5
       toler_ccr = ktoler/RP          ! AG.6,size independent
       toler_ccr2 = toler_ccr**2


	do jc=1,icfIJ
	if(ic.ne.jc)then

       jcgl=concf_ijf(jc)    

c dist**2 
       aa=null
       do k=1,3
       xyzdp(k)=xyzd(k)- cprobe_ijk(k,jcgl)
       aa = aa + UIJ(k)*xyzdp(k)
       end do !k

       dist=null
       do k=1,3
       dist = dist +(xyzdp(k)-UIJ(k)*aa)**2
       end do !k

       if(dist.le.(Hpmx*(one + toler_ccr)+toler_ccr2)) then
       WDYONRm(id)= .true.
c       write(*,'(a12,i5,2x,3f8.4,1x,f8.6,2x,L1,1x,L1)')'remove id=', id,
c     & (WDScAr(k,id),k=1,4),WDYON(id),WDYONRm(id)

       goto 101    ! next dot
       end if

       end if !ic.ne.jc
       end do !jc

       end if ! WDYON
 
101    continue
       end do !id 
       end if !nst nfin ge 1
       end do !ic
       end if !icfIJ ge 1

       return
       end
c
	  subroutine cxdotrfcut9(GG,dotch,dotchr,dotrot,dotstat,
     &	  dotCO,UIJ,Ds,RI,dotaw,nchmx)

c ndotloc - result:survived number of children dots
c 
c                                

           implicit none

	   integer nchmx
	   real*8 GG(3,3)
	   real*8 dotch(3,nchmx)    !initial Loc system dot coords
	   real*8 dotchr(3,nchmx)   !rotated Glob       dot coords
	   logical dotstat(nchmx)   !existence status 
	   logical dotrot(nchmx)    !rotation flag 
	   logical lastcut
	   real*8 dotCO(3),UIJ(3)
	   real*8 X(3),Y(3)
	   real*8 dotchg(3)
	   real*8 Ds,dotaw,RI

	 real*8 PI2,null,onehalf,one,two,three,four,big,small
	 common/CONSTANTI/PI2,null,onehalf,one,two,three,four,big,small
         real*8 nulls

	 real*8  toler_nb,toler_pr,toler_yon,toler_d,toler_cx,toler_ovr 
	 real*8  ktoler_yon
	 common/TOLERANCE/toler_nb,toler_pr,toler_yon,toler_d,
     &	 toler_cx,toler_ovr,ktoler_yon

	   integer i,j,k,k1,k2,nch
	   integer nnch
	   real*8 dnch,aa1,aa2
	   real*8 shif1,shif2,shif3
           integer print, iprint0,iprint1,iprintm
	   logical filter1,special(5)
	   integer dofilter
	   logical filter2,filter3,filter4

	   print =  -1
	   iprint0 = 0
	   iprint1 = 1
	   iprintm = -1
	   nnch =nchmx
c	   nnch = 1         !TEST

	   filter1=.true.    !TEST
	   filter2 = .false.   
	   filter3 = .false.
	   filter4 = .false.

	   dofilter = -1    !filter will be done twice, iteratively
	   dofilter = 0     !filter once

	   lastcut = .true. !proverjaem bsegda , t.k. ne znaem kakoi 

	   do k=1,5
	   special(k)=.false.
	   end do!k
c
	  if(print.ge.iprint0)then
	  write(*,*)'********************************************'
	  write(*,*)'in cxdot_refcut:'
	  end if 
	  if(print.ge.iprint0)then
	  write(*,*)'dotCO=',dotCO
	  write(*,*)'UIJ =',UIJ
	  write(*,*)'Ds, RI ',Ds, RI, dotaw
          end if

c check his position

	   dotaw = null         !dot completely deleted
	   do k=1,3
	   dotCO(k)= dotchr(k,1) !center
	   end do
	   do k=1,3
	   X(k)=null
	   end do !k

	   nch = 0

	   do i=1,nnch        ! loop children dots

	   if(.not.dotrot(i))then !rotate
	   do k=1,3
	   dotchr(k,i)=null
	   end do
           do k2=1,3
	   do k1=1,3
	   dotchr(k1,i)=dotchr(k1,i) + GG(k1,k2)*dotch(k2,i)
	   end do!k1
	   end do!k2

	   dotrot(i)=.true.
	   end if !rotate

	   if(dotstat(i))then  !dot exist and have to be cheked
	   aa1 = null
	   do k = 1,3
	   aa1=aa1 + dotchr(k,i)*UIJ(k)
	   end do!k

	  if(print.ge.iprint1)then
          write(*,*)'INITIAL LocXYZ child i =',i
	  write(*,*)'dotchg=',(dotch(k,i),k=1,3)
	  write(*,*)'ROTATED child i =',i
	  write(*,*)'dotchg=',(dotchr(k,i),k=1,3)
	  end if

	   if(aa1.gt.Ds )then ! delete this child 
	  if(print.ge.iprint1)then
	  write(*,*)'delete child i=',i, ' dotDs=',aa1
          end if

	   dotstat(i) = .false.

	   end if !delete
	   end if !exist

	   end do !i children

c
c
	  if(lastcut)then

	 
1005      continue               !dofilter loop

	  if(filter1)then
	  if(.not.dotstat(1) .and.(dotstat(2).and.dotstat(3)
     &       .and.dotstat(4).and.dotstat(5).and.dotstat(6)
     &       .and.dotstat(7).and.dotstat(8).and.dotstat(9)))then
	    do i = 2,9
	    dotstat(i)=.false.
	    end do !i
	    special(1)=.true.
	    end if

          if((.not.dotstat(1).and. .not.dotstat(9)).and.
     &       (dotstat(2).and.dotstat(3).and.dotstat(4)
     &       .and.dotstat(5).and.dotstat(6)
     &       .and.dotstat(7).and.dotstat(8)))then
	  dotstat(2)=.false.
	  dotstat(8)=.false.
	  special(2) = .true.
	  end if

	  if((.not.dotstat(1).and. .not.dotstat(3)).and.
     &       (dotstat(9).and.dotstat(2).and.dotstat(4)
     &       .and.dotstat(5).and.dotstat(6)
     &       .and.dotstat(7).and.dotstat(8)))then
	  dotstat(2)=.false.
	  dotstat(4)=.false.
	  special(3)=.true.
	  end if

	  if((.not.dotstat(1).and. .not.dotstat(7)).and.
     &       (dotstat(9).and.dotstat(2).and.dotstat(3)
     &       .and.dotstat(5).and.dotstat(6)
     &       .and.dotstat(4).and.dotstat(8)))then
	  dotstat(6)=.false.
	  dotstat(8)=.false.
	  special(4)=.true.
	  end if
 
	  if((.not.dotstat(1).and. .not.dotstat(5)).and.
     &       (dotstat(9).and.dotstat(2).and.dotstat(4)
     &       .and.dotstat(3).and.dotstat(6)
     &       .and.dotstat(7).and.dotstat(8)))then
	  dotstat(6)=.false.
	  dotstat(4)=.false.
	  special(5)=.true.
	  end if
	  end if !filter

	  if(filter2)then         !filter2
c b1:
	  if((.not.dotstat(1) .and. .not.dotstat(2) .and.
     &        .not.dotstat(6)) .and. 
     &       (dotstat(3).or.dotstat(4).or.dotstat(5)).and.
     &       (dotstat(7).or.dotstat(8).or.dotstat(9)))then
	  write(*,*)' b1: 1 2 6 - F'
	  do k=1,9
	  dotstat(k)=.false.
	  end do !k 
	  end if
c b2:
	  if((.not.dotstat(1) .and. .not.dotstat(4) .and.
     &        .not.dotstat(8)) .and. 
     &       (dotstat(2).or.dotstat(3).or.dotstat(9)).and.
     &       (dotstat(5).or.dotstat(6).or.dotstat(7)))then
	  write(*,*)' b2: 1 4 8 - F'
	  do k=1,9
	  dotstat(k)=.false.
	  end do !k 
	  end if
c b3:
	  if((.not.dotstat(1) .and. .not.dotstat(3) .and.
     &        .not.dotstat(7)) .and. 
     &       (dotstat(4).or.dotstat(5).or.dotstat(6)).and.
     &       (dotstat(2).or.dotstat(8).or.dotstat(9)))then
	  write(*,*)' b1: 1 3 7 - F'
	  do k=1,9
	  dotstat(k)=.false.
	  end do !k 
	  end if
c b4:
	  if((.not.dotstat(1) .and. .not.dotstat(5) .and.
     &        .not.dotstat(9)) .and. 
     &       (dotstat(3).or.dotstat(4).or.dotstat(2)).and.
     &       (dotstat(7).or.dotstat(8).or.dotstat(6)))then
	  write(*,*)' b1: 1 5 9 - F'
	  do k=1,9
	  dotstat(k)=.false.
	  end do !k 
	  end if

	  end if !filter2

	  if(filter3)then
c b5:
	  if((.not.dotstat(3) .and. .not.dotstat(5)).and.
     &       dotstat(4).and.
     &       (dotstat(1).or.dotstat(2).or.dotstat(6).or.
     &        dotstat(7).or.dotstat(8).or.dotstat(9)))then 
	     dotstat(4)=.false.
	  end if
c b6:
	  if((.not.dotstat(7) .and. .not.dotstat(5)).and.
     &       dotstat(6).and.
     &       (dotstat(1).or.dotstat(4).or.dotstat(8).or.
     &        dotstat(2).or.dotstat(3).or.dotstat(9)))then 
	     dotstat(6)=.false.
	  end if
c b7:
	  if((.not.dotstat(7) .and. .not.dotstat(9)).and.
     &       dotstat(8).and.
     &       (dotstat(1).or.dotstat(2).or.dotstat(6).or.
     &        dotstat(3).or.dotstat(4).or.dotstat(5)))then 
	     dotstat(8)=.false.
	  end if
c b8:
	  if((.not.dotstat(3) .and. .not.dotstat(9)).and.
     &       dotstat(2).and.
     &       (dotstat(1).or.dotstat(4).or.dotstat(8).or.
     &        dotstat(7).or.dotstat(5).or.dotstat(6)))then 
	     dotstat(2)=.false.
	  end if
	
	  end if !filter3

	  if(filter4)then
c b9:
	  if((.not.dotstat(1) .and. .not.dotstat(2)))then
	  dotstat(3)=.false.
	  dotstat(9)=.false.
	  end if
c b10:
	  if((.not.dotstat(1) .and. .not.dotstat(4)))then
	  dotstat(3)=.false.
	  dotstat(5)=.false.
	  end if
c b11:
	  if((.not.dotstat(1) .and. .not.dotstat(6)))then
	  dotstat(7)=.false.
	  dotstat(5)=.false.
	  end if
c b12:
	  if((.not.dotstat(1) .and. .not.dotstat(8)))then
	  dotstat(7)=.false.
	  dotstat(9)=.false.
	  end if

	  end if !filter4

	  dofilter = dofilter + 1
c	  if(dofilter.le.0)goto 1005


             nch = 0
	   do k=1,3
	   X(k)=null
	   end do !k

	   do i = 1,nnch
	   if(dotstat(i))then
	   nch=nch + 1
           do k=1,3
	   X(k)= X(k) + dotchr(k,i)
	   end do !k
	   end if
	   end do !i dots

c result
	   if(nch.gt.0)then
	   dotaw = dfloat(nch)/dfloat(nnch)

	   call DVNORM(X,Y)

	   do k=1,3
	   dotCO(k)=RI*Y(k)     
	   end do !k
	   end if

	   if(nch.eq.0)then
	   dotaw = null         !dot completely deleted
	   do k=1,3
	   dotCO(k)= dotchr(k,1) !center
	   end do
	   
	   end if !nch gt 0
c
	  if(print.ge.iprintm)then
	  do k=1,5
	  if(special(k))then
	  write(*,*)'res cxdot_refcut:special',special,'  nch ',nch
	  end if
	  end do
	  end if

	  end if !last cut

	  if(print.ge.iprint0)then
	  write(*,*)'nch =',nch
	  write(*,*)'dotCO=',dotCO
	  write(*,*)'dotstat=',(dotstat(k),k=1,nnch),'  dotaw =',dotaw
          end if

          return
	  end
c
	  subroutine cxdotrfcut5(GG,dotch,dotchr,dotrot,dotstat,
     &	  dotCO,UIJ,Ds,RI,dotaw,nchmx,lastcut)


           implicit none

	   integer nchmx
	   real*8 GG(3,3)
	   real*8 dotch(3,nchmx)    !initial Loc system dot coords
	   real*8 dotchr(3,nchmx)   !rotated Glob       dot coords
	   logical dotstat(nchmx)   !existence status 
	   logical dotrot(nchmx)    !rotation flag 
	   logical lastcut
	   real*8 dotCO(3),UIJ(3)
	   real*8 X(3),Y(3)
	   real*8 dotchg(3)
	   real*8 Ds,dotaw,RI

	 real*8 PI2,null,onehalf,one,two,three,four,big,small
	 common/CONSTANTI/PI2,null,onehalf,one,two,three,four,big,small
         real*8 nulls

	 real*8  toler_nb,toler_pr,toler_yon,toler_d,toler_cx,toler_ovr 
	 real*8  ktoler_yon
	 common/TOLERANCE/toler_nb,toler_pr,toler_yon,toler_d,
     &	 toler_cx,toler_ovr,ktoler_yon

	   integer i,j,k,k1,k2,nch
	   integer nnch
	   real*8 dnch,aa1,aa2
	   real*8 shif1,shif2,shif3
           integer print, iprint0,iprint1,iprintm
	   logical filter1

	   print =  -1
	   iprint0 = 0
	   iprint1 = 1
	   iprintm = -1
	   nnch =nchmx

	   filter1=.false.    !TEST

	   lastcut = .true. !proverjaem bsegda , t.k. ne znaem kakoi 

	  if(print.ge.iprint0)then
	  write(*,*)'********************************************'
	  write(*,*)'in cxdot_refcut:'
	  end if 
	  if(print.ge.iprint0)then
	  write(*,*)'dotCO=',dotCO
	  write(*,*)'UIJ =',UIJ
	  write(*,*)'Ds, RI ',Ds, RI, dotaw
          end if


	   do i=1,nnch        ! loop children dots

	   if(.not.dotrot(i))then !rotate
	   do k=1,3
	   dotchr(k,i)=null
	   end do
           do k2=1,3
	   do k1=1,3
	   dotchr(k1,i)=dotchr(k1,i) + GG(k1,k2)*dotch(k2,i)
	   end do!k1
	   end do!k2

	   dotrot(i)=.true.
	   end if !rotate

	   if(dotstat(i))then  !dot exist and have to be cheked
	   aa1 = null
	   do k = 1,3
	   aa1=aa1 + dotchr(k,i)*UIJ(k)
	   end do!k

	  if(print.ge.iprint1)then
          write(*,*)'INITIAL LocXYZ child i =',i
	  write(*,*)'dotchg=',(dotch(k,i),k=1,3)
	  write(*,*)'ROTATED child i =',i
	  write(*,*)'dotchg=',(dotchr(k,i),k=1,3)
	  end if

	   if(aa1.gt.Ds)then ! Delete this child 
	  if(print.ge.iprint1)then
	  write(*,*)'delete child i=',i, ' dotDs=',aa1
          end if

	   dotstat(i) = .false.

	   end if !delete
	   end if !exist

	   end do !i children

1005      continue

c
	 
	  if(filter1)then

	  if(.not.dotstat(1).and.(dotstat(2).and.dotstat(3)
     &       .and.dotstat(4).and.dotstat(5)))then
	  do k = 2,5
	  dotstat(k)=.false.
	  end do!k
	  end if !delete all childrens

	  if((.not.dotstat(2) .and. .not.dotstat(4)).and.
     &    (dotstat(3).and.dotstat(5)))then
	    dotstat(3)=.false.
	    dotstat(5)=.false.
	    end if

	  if((.not.dotstat(3) .and. .not.dotstat(5)).and.
     &    (dotstat(2).and.dotstat(4)))then
	    dotstat(2)=.false.
	    dotstat(4)=.false.
	    end if
	  end if !filter1


             nch = 0
	   do k=1,3
	   X(k)=null
	   end do !k

	   do i = 2,nnch       

	   if(dotstat(i))then
	   nch=nch + 1
           do k=1,3
	   X(k)= X(k) + dotchr(k,i)
	   end do !k
	   end if
	   end do !i dots

c result
	   if(nch.gt.0)then
	   dotaw = dfloat(nch)/dfloat(nnch - 1)

	   call DVNORM(X,Y)

	   do k=1,3
	   dotCO(k)=RI*Y(k)      
	   end do !k
	   end if

	   if(nch.eq.0)then
	   dotaw = null         
	   do k=1,3
	   dotCO(k)= dotchr(k,1) 
	   end do
	   
	   end if !nch gt 0
c

	  if(print.ge.iprint0)then
	  write(*,*)'nch =',nch
	  write(*,*)'dotCO=',dotCO
	  write(*,*)'dotstat=',(dotstat(k),k=1,nnch),'  dotaw =',dotaw
          end if

          return
	  end
