#!/bin/sh
# $Date$
# $Revision$

##########################################################################
# test_energy.sh - Tests the ECEPPAK program's "ENERGY" functionality
#                  (Usually called by the test_eceppak.sh script)
#
# History:
#  gtoll - 9/8/95 - created
#
##########################################################################

# Variables to customize this script's behavior
DIFF2USE=${DIFF2USE:-diff}	# Diff program to use to compare files
DIFFSUFFIX=${DIFFSUFFIX:-diff}	# Suffix to indicate files produced by diff
NUMPROCS=${NUMPROCS:-1}		# Number of processors to use
REFDIR=${REFDIR:-Ref}		# Directory where reference files are
RUNSCRIPT=${RUNSCRIPT:-recepp.s} # Script used to run the ECEPPAK program
TESTDIR="$ECEPP_ROOT/Test"	# Directory containing the tests

# Do NOT edit below this line
##########################################################################

# By default, all diff output will be copied into the directory from which 
# the tests are being run. If $DIFFDEST is already set (e.g. by the 
# test_eceppak.sh script), leave it alone. 
DIFFDEST=${DIFFDEST:-`pwd`}

# Runtype specific variables
RUNTYPE="ENERGY"

##########################################################################
# Function Declarations

# Compare each output file to its corresponding reference file
CheckOutput()
{
   for file in $*
   do
      if [ -r $file ]
      then
         $DIFF2USE $file Ref > $DIFFDEST/$file.$DIFFSUFFIX
      else
         echo "*** Warning: Output file $file not found. ***"
      fi
   done
}

##########################################################################

#
# Main Body
#

# Better make sure everything's set properly before we begin...
if [ -z "$ECEPP_ROOT" ]
then
   echo "Error: Environment variable \$ECEPP_ROOT not set."
   exit 1
fi
if [ -z "$ECEPP_ARCH" ]
then
   echo "Error: Environment variable \$ECEPP_ARCH not set."
   exit 1
fi

# Run the tests:
echo
echo "Testing $RUNTYPE functionality..."

# ala_map Test
cd "$TESTDIR/$RUNTYPE/ala_map"
$RUNSCRIPT ENERGY ala_map ALAMAP dummy dummy $NUMPROCS
CheckOutput main_out.ALAMAP

# avian Test
cd "$TESTDIR/$RUNTYPE/avian"
$RUNSCRIPT $RUNTYPE avian AVIAN dummy dummy $NUMPROCS
CheckOutput main_out.AVIAN

# cala6 Test
cd "$TESTDIR/$RUNTYPE/cala6"
$RUNSCRIPT $RUNTYPE cala6 CALA6 dummy dummy $NUMPROCS
CheckOutput main_out.CALA6

# cys1 Test
cd "$TESTDIR/$RUNTYPE/cys1"
$RUNSCRIPT $RUNTYPE cys1 CYS1 dummy dummy $NUMPROCS
CheckOutput main_out.CYS1

# enk_sol Test
cd "$TESTDIR/$RUNTYPE/enk_sol"
$RUNSCRIPT $RUNTYPE enk_sol ENK_SOL dummy dummy $NUMPROCS
CheckOutput main_out.ENK_SOL

# enk_sp Test
cd "$TESTDIR/$RUNTYPE/enk_sp"
$RUNSCRIPT $RUNTYPE enk_sp ENK_SP dummy dummy $NUMPROCS
CheckOutput main_out.ENK_SP

# pp Test
cd "$TESTDIR/$RUNTYPE/pp"
$RUNSCRIPT $RUNTYPE pp PP pp dummy $NUMPROCS
CheckOutput main_out.PP

# three_let Test
cd "$TESTDIR/$RUNTYPE/three_let"
$RUNSCRIPT $RUNTYPE three_let THREE_LET dummy dummy $NUMPROCS
CheckOutput main_out.THREE_LET

echo "$RUNTYPE testing completed."
echo
exit 0
