#!/bin/sh
# $Date$
# $Revision$

##########################################################################
# test_minimize.sh - Tests the ECEPPAK program's "EDMC" functionality
#                  (Usually called by the test_eceppak.sh script)
#
# History:
#  gtoll - 9/8/95 - created
#
##########################################################################

# Variables to customize this script's behavior
DIFF2USE=${DIFF2USE:-diff}	# Diff program to use to compare files
DIFFSUFFIX=${DIFFSUFFIX:-diff}	# Suffix to indicate files produced by diff
NUMPROCS=${NUMPROCS:-1}		# Number of processors to use
REFDIR=${REFDIR:-Ref}		# Directory where reference files are
RUNSCRIPT=${RUNSCRIPT:-recepp.s} # Script used to run the ECEPPAK program
TESTDIR="$ECEPP_ROOT/Test"	# Directory containing the tests

# Do NOT edit below this line
##########################################################################

# By default, all diff output will be copied into the directory from which 
# the tests are being run. If $DIFFDEST is already set (e.g. by the 
# test_eceppak.sh script), leave it alone. 
DIFFDEST=${DIFFDEST:-`pwd`}

# Runtype specific variables
RUNTYPE="MINIMIZE"

##########################################################################
# Function Declarations

# Compare each output file to its corresponding reference file
CheckOutput()
{
   for file in $*
   do
      if [ -r $file ]
      then
         $DIFF2USE $file Ref > $DIFFDEST/$file.$DIFFSUFFIX
      else
         echo "*** Warning: Output file $file not found. ***"
      fi
   done
}

##########################################################################

#
# Main Body
#

# Better make sure everything's set properly before we begin...
if [ -z "$ECEPP_ROOT" ]
then
   echo "Error: Environment variable \$ECEPP_ROOT not set."
   exit 1
fi
if [ -z "$ECEPP_ARCH" ]
then
   echo "Error: Environment variable \$ECEPP_ARCH not set."
   exit 1
fi

# Run the tests:
echo
echo "Testing $RUNTYPE functionality..."

# CPEP Test
cd "$TESTDIR/$RUNTYPE/CPEP"
$RUNSCRIPT $RUNTYPE CPEP CPEPout CPEP dummy $NUMPROCS
CheckOutput main_out.CPEPout

# mpa1ot Test
cd "$TESTDIR/$RUNTYPE/mpa1ot"
$RUNSCRIPT $RUNTYPE mpa1ot MPA1OT dummy dummy $NUMPROCS
CheckOutput main_out.MPA1OT outo.MPA1OT sumsl.MPA1OT

echo "$RUNTYPE testing completed."
echo
exit 0
