#!/bin/sh
# $Date$
# $Revision$

##########################################################################
# test_rms_fit.sh - Tests the ECEPPAK program's "RMS_FIT" functionality
#                  (Usually called by the test_eceppak.sh script)
#
# History:
#  gtoll - 9/8/95 - created
#
##########################################################################

# Variables to customize this script's behavior
DIFF2USE=${DIFF2USE:-diff}	# Diff program to use to compare files
DIFFSUFFIX=${DIFFSUFFIX:-diff}	# Suffix to indicate files produced by diff
NUMPROCS=${NUMPROCS:-1}		# Number of processors to use
REFDIR=${REFDIR:-Ref}		# Directory where reference files are
RUNSCRIPT=${RUNSCRIPT:-recepp.s} # Script used to run the ECEPPAK program
TESTDIR="$ECEPP_ROOT/Test"	# Directory containing the tests

# Do NOT edit below this line
##########################################################################

# By default, all diff output will be copied into the directory from which 
# the tests are being run. If $DIFFDEST is already set (e.g. by the 
# test_eceppak.sh script), leave it alone. 
DIFFDEST=${DIFFDEST:-`pwd`}

# Runtype specific variables
RUNTYPE="RMS_FIT"

##########################################################################
# Function Declarations

# Compare each output file to its corresponding reference file
CheckOutput()
{
   for file in $*
   do
      if [ -r $file ]
      then
         $DIFF2USE $file Ref > $DIFFDEST/$file.$DIFFSUFFIX
      else
         echo "*** Warning: Output file $file not found. ***"
      fi
   done
}

##########################################################################

#
# Main Body
#

# Better make sure everything's set properly before we begin...
if [ -z "$ECEPP_ROOT" ]
then
   echo "Error: Environment variable \$ECEPP_ROOT not set."
   exit 1
fi
if [ -z "$ECEPP_ARCH" ]
then
   echo "Error: Environment variable \$ECEPP_ARCH not set."
   exit 1
fi

# Run the tests:
echo
echo "Testing $RUNTYPE functionality..."

# ala_rms1 Test
cd "$TESTDIR/$RUNTYPE/ala_rms1"
$RUNSCRIPT $RUNTYPE ala_rms1 ALA_RMS1 ala_rms ala_HELIX $NUMPROCS
CheckOutput main_out.ALA_RMS1 chefol.ALA_RMS1 rgyr.ALA_RMS1 xray.ala_HELIX

# ala_rms2 Test
cd "$TESTDIR/$RUNTYPE/ala_rms2"
$RUNSCRIPT $RUNTYPE ala_rms2 ALA_RMS2 ala135 ala_HELIX $NUMPROCS
CheckOutput main_out.ALA_RMS2 chefol.ALA_RMS2 rgyr.ALA_RMS2

echo "$RUNTYPE testing completed."
echo
exit 0
