#!/bin/sh
# $Date$
# $Revision$

##########################################################################
# test_eceppak.sh - Tests the ECEPPAK program's functionality
#		(Basically this is just a front-end to all of the
#		 different test_<runtype>.sh scripts.)
#
# History:
#  gtoll - 9/8/95 - created
#
##########################################################################

# Variables to customize this script's behavior
echo $ECEPP_ROOT
echo
DIFF2USE="diff"                 # Diff program to use to compare files
export DIFF2USE
DIFFSUFFIX="diff"               # Suffix to indicate files produced by diff
export DIFFSUFFIX
REFDIR="Ref"                    # Directory where reference files are
export REFDIR
RUNSCRIPT="${ECEPP_ROOT}/bin/recepp-${ECEPP_ARCH}.s"    # Script used to run the ECEPPAK program
export RUNSCRIPT
TESTDIR="$ECEPP_ROOT/Test"      # Directory containing the tests
export TESTDIR

# Do NOT edit below this line
##########################################################################


PGMNAME=`basename $0`
TESTDIR="$ECEPP_ROOT/Test"
RUNTYPELIST="BOUNDS CHECKGRAD EDMC ENERGY MINIMIZE RMS_FIT VTF"

if [ -z "$ECEPP_ROOT" ]
then
   echo "Error: Environment variable \$ECEPP_ROOT not set."
   exit 1
fi

# Initialize some trap handlers to do cleanup work
TrapSig0()
{
   # No need for any output here since sig0 means our shell just exited
   find . -name ECEPP_ROOTA -exec rm {} \;
   exit 2
}

TrapSig1()
{
   echo "Received signal SIGHUP.  Terminating..."
   find . -name ECEPP_ROOTA -exec rm {} \;
   exit 2
}

TrapSig2()
{
   echo "Received signal SIGINT.  Terminating..."
   find . -name ECEPP_ROOTA -exec rm {} \;
   exit 2
}

# Now we better initialize the trap vectors
trap TrapSig0 0
trap TrapSig1 1
trap TrapSig2 2


ShowUsage()
{
   echo "Usage: $PGMNAME [-h] [-d <dirname>] [-n <numprocs>] [list of runtypes]"
}

ShowOptions()
{
   echo "Options:"
   sed -n '/^#GetFlags/,/^#EndGetFlag/ p' $0 | \
   sed -n 's/^.*\([-+][0-9A-Za-z*])\)/ \1/p' | sed 's/ #//'
}

#GetFlags - Do NOT delete this line
while [ $# -gt 0 ]
   do
      case "$1" in
         -d) # Specify destination directory for diff files
            shift
            DIFFDEST=`pwd`/$1; export DIFFDEST
            echo "All diff files will be placed in $DIFFDEST."
            shift
            ;;
         -h) # "Help" - this message
            ShowUsage
            ShowOptions
            echo
            exit 0
            ;;
         -n) # Specify number of processors to use
            shift
            NUMPROCS="$1"; export NUMPROCS
            echo "$NUMPROCS processors will be used."
            shift
            ;;
          *)
            capname=`echo $1 | tr [a-z] [A-Z]`
            echo $RUNTYPELIST | grep -w "$capname" > /dev/null
            if [ "$?" -eq "0" ] 
            then
               UserRunList="$UserRunList $capname"
            else
               echo "Error: Invalid argument specified - \"$1\" "
               exit 1
            fi
            shift
            ;;
       esac
   done
#EndGetFlags - Do NOT delete this line


# If the user has specified certain tests to run, then only run those tests 
if [ -n "$UserRunList" ]
then
   RUNTYPELIST="$UserRunList"
fi

for rtype in $RUNTYPELIST
do
   name=`echo $rtype | tr [A-Z] [a-z]`
   $TESTDIR/$rtype/test_$name.sh
done

echo "$PGMNAME completed"
exit 0
